"""
Tests the interface.renew_vessels and interface.renew_all_vessels calls.
"""

# The seattlegeni testlib must be imported first.
from seattlegeni.tests import testlib

from seattlegeni.tests import mocklib

from seattlegeni.common.api import maindb

from seattlegeni.common.exceptions import *

from seattlegeni.website.control import interface

from seattlegeni.website.tests import testutil

import datetime
import unittest





mocklib.mock_lockserver_calls()





class SeattleGeniTestCase(unittest.TestCase):


  def setUp(self):
    # Setup a fresh database for each test.
    testlib.setup_test_db()



  def tearDown(self):
    # Cleanup the test database.
    testlib.teardown_test_db()



  def test_renew_vessels_insufficient_vessel_credits(self):
    
    # Create a user who will be doing the acquiring.
    user = maindb.create_user("testuser", "password", "example@example.com", "affiliation", "1 2", "2 2 2", "3 4")
    userport = user.usable_vessel_port
    
    vesselcount = 4
    
    # Have every vessel acquisition to the backend request succeed.
    calls_results = [True] * vesselcount
    mocklib.mock_backend_acquire_vessel(calls_results)
    
    testutil.create_nodes_on_different_subnets(vesselcount, [userport])
    
    # Acquire all of the vessels the user can acquire.
    vessel_list = interface.acquire_vessels(user, vesselcount, 'rand')
    
    # Decrease the user's vessel credits to one less than the number of vessels
    # they have acquired.
    user.free_vessel_credits = 0
    user.save()
    
    func = interface.renew_vessels
    args = (user, vessel_list)
    self.assertRaises(InsufficientUserResourcesError, func, *args)      
  
    func = interface.renew_all_vessels
    args = (user,)
    self.assertRaises(InsufficientUserResourcesError, func, *args)   



  def test_renew_some_of_users_vessel(self):
    
    # Create a user who will be doing the acquiring.
    user = maindb.create_user("testuser", "password", "example@example.com", "affiliation", "1 2", "2 2 2", "3 4")
    userport = user.usable_vessel_port
    
    vesselcount = 4
    
    # Have every vessel acquisition to the backend request succeed.
    calls_results = [True] * vesselcount
    mocklib.mock_backend_acquire_vessel(calls_results)
    
    testutil.create_nodes_on_different_subnets(vesselcount, [userport])
    
    # Acquire all of the vessels the user can acquire.
    vessel_list = interface.acquire_vessels(user, vesselcount, 'rand')
    
    renew_vessels_list = vessel_list[:2]
    not_renewed_vessels_list = vessel_list[2:]
    
    interface.renew_vessels(user, renew_vessels_list)
  
    now = datetime.datetime.now()
    timedelta_oneday = datetime.timedelta(days=1)
  
    for vessel in renew_vessels_list:
      self.assertTrue(vessel.date_expires - now > timedelta_oneday)

    for vessel in not_renewed_vessels_list:
      self.assertTrue(vessel.date_expires - now < timedelta_oneday)



  def test_renew_vessels_dont_belong_to_user(self):
    
    # Create a user who will be doing the acquiring.
    user = maindb.create_user("testuser", "password", "example@example.com", "affiliation", "1 2", "2 2 2", "3 4")
    userport = user.usable_vessel_port
    
    # Create a second user.
    user2 = maindb.create_user("user2", "password", "user2@example.com", "affiliation", "1 2", "2 2 2", "3 4")
    
    vesselcount = 4
    
    # Have every vessel acquisition to the backend request succeed.
    calls_results = [True] * vesselcount
    mocklib.mock_backend_acquire_vessel(calls_results)
    
    testutil.create_nodes_on_different_subnets(vesselcount, [userport])
    
    # Acquire all of the vessels the user can acquire.
    vessel_list = interface.acquire_vessels(user, vesselcount, 'rand')
    
    release_vessel = vessel_list[0]
    interface.release_vessels(user, [release_vessel])
    
    # Manually fiddle with one of the vessels to make it owned by user2.
    user2_vessel = vessel_list[1]
    user2_vessel.acquired_by_user = user2
    user2_vessel.save()
    
    # Try to renew all of the originally acquired vessels, including the ones
    # that were released. We expect these to just be ignored.
    interface.renew_vessels(user, vessel_list)
  
    # Get fresh vessel objects that reflect the renewal.
    remaining_vessels = interface.get_acquired_vessels(user)
    release_vessel = maindb.get_vessel(release_vessel.node.node_identifier, release_vessel.name)
    user2_vessel = maindb.get_vessel(user2_vessel.node.node_identifier, user2_vessel.name)
   
    now = datetime.datetime.now()
    timedelta_oneday = datetime.timedelta(days=1)
  
    # Ensure that the vessels the user still has were renewed but that the ones
    # the user released were ignored (not renewed).
    for vessel in remaining_vessels:
      self.assertTrue(vessel.date_expires - now > timedelta_oneday)

    self.assertTrue(user2_vessel.date_expires - now < timedelta_oneday)
    
    self.assertEqual(release_vessel.date_expires, None)
  




def run_test():
  unittest.main()



if __name__ == "__main__":
  run_test()



"""
<Program>
  lockserver.py

<Started>
  29 June 2009

<Author>
  Justin Samuel

<Purpose>
  This is the API that should be used to interact with the Lockserver.
  Functions in this module are the only way that other code should interact
  with the Lockserver.
   
  There is no provided function to unlock both user locks and node lock
  in a single call even though that is currently supported by the Lockserver.
  This can be added later, if needed.

  There are separate calls for locking/unlocking a single user or node
  and locking/unlocking multiple users or nodes. This is to make client
  code easier to read and less error-prone (e.g. forgetting to pass a
  list that contains a single name and instead directly passing the
  string that has the lock name).

  For details about the locking rules, see the module comments in
  lockserver_daemon.py.


<Usage>
  # Create a lockserver handle.
  lockserver_handle = create_lockserver_handle()

  # Lock request blocks until lock is obtained.
  lock_user(lockserver_handle, 'bob')

  # Release the user lock in a finally block.
  try:
    ...
    # Lock request blocks until lock is obtained.
    lock_multiple_nodes(lockserver_handle, ['123', '456'])
  
    # Release the node locks in a finally block.
    try:
      ...
    
    finally:
      # Unlock requests do not block.
      unlock_multiple_nodes(lockserver_handle, ['123', '456'])

  finally:
    # If unlock_user threw an exception here, we wouldn't destroy the lockserver
    # handle. That is not good, but it's not the critical error that failing to
    # release the user lock would be.
    unlock_user(lockserver_handle, 'bob')
  
    # Destroy the lockserver handle (don't forget to do this!)
    destroy_lockserver_handle(lockserver_handle)

"""

import datetime
import socket
import traceback
import xmlrpclib

from seattlegeni.common.exceptions import *

from seattlegeni.common.util.decorators import log_function_call





# The default lockserver url to use.
LOCKSERVER_URL = "http://127.0.0.1:8010"

# Constants to prevent unnoticed typos in the code below.
REQUEST_TYPE_LOCK = 'lock'
REQUEST_TYPE_UNLOCK = 'unlock'





@log_function_call
def create_lockserver_handle(lockserver_url=LOCKSERVER_URL):
  """
  <Purpose>
    Create a handle for communication with a lockserver. This is a required
    step before a client calls any of the (un)lock_* functions in this module.
    Note that a client should only create a single handle across multiple
    lock requests. This may require passing the handle around in function calls
    so that helper functions that create locks can use the same handle.
    It is very important that the lockserver handle be later destroyed by
    calling destroy_lockserver_handle().
  <Arguments>
    lockserver_url
      The url of the lockserver. Defaults to the lockserver running locally on
      its usual port.
  <Exceptions>
    ProgrammerError
    InternalError
      If the lockserver can't be communicated with.
  <Side Effects>
    Starts a session with the lockserver.
  <Returns>
    A lockserver handle.
  """
  
  lockserver_handle = {}
  lockserver_handle["proxy"] = xmlrpclib.ServerProxy(lockserver_url)
  
  try:
    lockserver_handle["session_id"] = lockserver_handle["proxy"].StartSession()
  except xmlrpclib.Fault:
    raise ProgrammerError("The lockserver rejected the request: " + traceback.format_exc())
  except xmlrpclib.ProtocolError:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())
  except socket.error:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())
  
  return lockserver_handle





def destroy_lockserver_handle(lockserver_handle):
  """
  <Purpose>
    Destroys a lockserver handle previously created with
    create_lockserver_handle().
  <Arguments>
    lockserver_handle
      The lockserver handle to destroy.
  <Exceptions>
    ProgrammerError
    InternalError
      If the lockserver can't be communicated with.
  <Side Effects>
    Ends the session on the lockserver.
  <Returns>
    None.
  """
  
  try:
    lockserver_handle["proxy"].EndSession(lockserver_handle["session_id"])
  except xmlrpclib.Fault:
    raise ProgrammerError("The lockserver rejected the request: " + traceback.format_exc())
  except xmlrpclib.ProtocolError:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())
  except socket.error:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())





def lock_user(lockserver_handle, username):
  """
  <Purpose>
    Obtains a user lock.
  <Arguments>
    lockserver_handle
      The lockserver handle whose session the lock will be obtained under.
    username
      The username string of the user to obtain the lock on.
  <Exceptions>
    ProgrammerError
    InternalError
      If the lockserver can't be communicated with.
  <Side Effects>
    Blocks until the lock is obtained.
  <Returns>
    None.
  """
  _perform_lock_request(REQUEST_TYPE_LOCK, lockserver_handle, user_list=[username])





def unlock_user(lockserver_handle, username):
  """
  <Purpose>
    Release a user lock previously obtained with the same lockserver_handle.
  <Arguments>
    lockserver_handle
      The lockserver handle whose session the lock will be released under.
    username
      The username string of the user to release the lock of.
  <Exceptions>
    ProgrammerError
    InternalError
      If the lockserver can't be communicated with.
  <Side Effects>
    Releases the lock.
  <Returns>
    None.
  """
  _perform_lock_request(REQUEST_TYPE_UNLOCK, lockserver_handle, user_list=[username])





def lock_multiple_users(lockserver_handle, username_list):
  """
  <Purpose>
    Obtains locks on multiple users.
  <Arguments>
    lockserver_handle
      The lockserver handle whose session the locks will be obtained under.
    username_list
      The list of username strings of the users to obtain locks on.
  <Exceptions>
    ProgrammerError
    InternalError
      If the lockserver can't be communicated with.
  <Side Effects>
    Blocks until all requested locks are obtained.
  <Returns>
    None.
  """
  _perform_lock_request(REQUEST_TYPE_LOCK, lockserver_handle, user_list=username_list)





def unlock_multiple_users(lockserver_handle, username_list):
  """
  <Purpose>
    Release user locks previously obtained with the same lockserver_handle.
  <Arguments>
    lockserver_handle
      The lockserver handle whose session the locks will be released under.
    username_list
      The list of username strings of the users to release the locks of.
  <Exceptions>
    ProgrammerError
    InternalError
      If the lockserver can't be communicated with.
  <Side Effects>
    Releases the locks.
  <Returns>
    None.
  """
  _perform_lock_request(REQUEST_TYPE_UNLOCK, lockserver_handle, user_list=username_list)





def lock_node(lockserver_handle, node_id):
  """
  See the comments for the user lock functions as well as the locking rules
  described in the module comments.
  """
  _perform_lock_request(REQUEST_TYPE_LOCK, lockserver_handle, node_list=[node_id])





def unlock_node(lockserver_handle, node_id):
  """
  See the comments for the user lock functions as well as the locking rules
  described in the module comments.
  """
  _perform_lock_request(REQUEST_TYPE_UNLOCK, lockserver_handle, node_list=[node_id])





def lock_multiple_nodes(lockserver_handle, node_id_list):
  """
  See the comments for the user lock functions as well as the locking rules
  described in the module comments.
  """
  _perform_lock_request(REQUEST_TYPE_LOCK, lockserver_handle, node_list=node_id_list)





def unlock_multiple_nodes(lockserver_handle, node_id_list):
  """
  See the comments for the user lock functions as well as the locking rules
  described in the module comments.
  """
  _perform_lock_request(REQUEST_TYPE_UNLOCK, lockserver_handle, node_list=node_id_list)



  
  
def _perform_lock_request(request_type, lockserver_handle, user_list=None, node_list=None):
  """
  A helper function that does the actual lock or unlock calls to the lockserver.
  """

  session_id = lockserver_handle["session_id"]
  lockdict = {}
  
  if user_list is not None:
    lockdict["user"] = user_list
  if node_list is not None:
    lockdict["node"] = node_list
  
  if request_type is REQUEST_TYPE_LOCK:
    request_func = lockserver_handle["proxy"].AcquireLocks
  elif request_type is REQUEST_TYPE_UNLOCK:
    request_func = lockserver_handle["proxy"].ReleaseLocks
  else:
    raise ProgrammerError("Invalid lock request type specified: " + str(request_type))
    
  try:
    request_func(session_id, lockdict)
  except xmlrpclib.Fault:
    raise ProgrammerError("The lockserver rejected the request: " + traceback.format_exc())
  except xmlrpclib.ProtocolError:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())
  except socket.error:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())





def get_status(lockserver_url=LOCKSERVER_URL):
  """
  <Purpose>
    Query the lockserver for its status. This function is for monitoring
    purposes and will never need to be called during normal operation.
  <Arguments>
    lockserver_url
      The url of the lockserver to call GetStatus on. Defaults to LOCKSERVER_URL.
  <Exceptions>
    ProgrammerError
    InternalError
      If the lockserver can't be communicated with.
  <Side Effects>
    None
  <Returns>
    See the documentation for the GetStatus() call in lockserver_daemon.py.
  """
  
  proxy = xmlrpclib.ServerProxy(lockserver_url)
  try:
    statusdict = proxy.GetStatus()
    
    # Convert the locktimelist datetime object string representations back into
    # datetime objects. Code borrowed from:
    # http://docs.python.org/library/xmlrpclib.html#datetime-objects
    locktimelist = []
    for locktimeitem in statusdict["locktimelist"]:
      # Convert the ISO8601 string to a datetime object.
      print locktimeitem[1].value
      converteddatetime = datetime.datetime.strptime(locktimeitem[1].value, "%Y%m%dT%H:%M:%S")
      locktimelist.append((locktimeitem[0], converteddatetime))
    statusdict["locktimelist"] = locktimelist
    
    return statusdict

  except xmlrpclib.Fault:
    raise ProgrammerError("The lockserver rejected the request: " + traceback.format_exc())
  except xmlrpclib.ProtocolError:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())
  except socket.error:
    raise InternalError("Unable to communicate with the lockserver: " + traceback.format_exc())
    



#!/usr/bin/env python
"""
<Purpose>
  This script is used to immediately stop everything running on a node.

  Usage:
    ./stop_all_vessels_on_node.py "<node_id>"

  Note that the node_id contains a space, so you need to put it in quotes.

  You will be given status information as the process is completed.
"""

import sys
import time

from seattlegeni.common.api import lockserver
from seattlegeni.common.api import maindb
from seattlegeni.common.util import log

from seattlegeni.common.exceptions import *


# Set the log level high enough so that we don't produce a bunch of logging
# output due to the logging decorators.
initial_log_level = log.loglevel
log.set_log_level(log.LOG_LEVEL_INFO)



def stop_all_vessels_on_node(node_id):

  try:
    node = maindb.get_node(node_id)
  except DoesNotExistError:
    print "No such node"
    sys.exit(1)

  if not node.is_active:
    print "This node is marked as inactive, thus the backend will not try to clean up vessels."
    sys.exit(0)

  if node.is_broken:
    print "This node is marked as broken, thus the backend will not try to clean up vessels."
    sys.exit(0)

  vessels_on_node = maindb.get_vessels_on_node(node)
  if not vessels_on_node:
    print "No vessels on node."
    sys.exit(0)

  lockserver_handle = lockserver.create_lockserver_handle()
  try:
    print "Indicating to the backend to release/reset all %s vessels." % len(vessels_on_node)
    lockserver.lock_node(lockserver_handle, node_id)
    try:
      for vessel in vessels_on_node:
        maindb.record_released_vessel(vessel)
    finally:
      lockserver.unlock_node(lockserver_handle, node_id)

    print "Releases indicated. Monitoring db to see if the backend cleaned them up."
    while True:
      for vessel in vessels_on_node[:]:
        updated_vessel = maindb.get_vessel(node_id, vessel.name)
        if updated_vessel.is_dirty:
          print "Vessel %s has not been cleaned up yet." % updated_vessel
        else:
          print "Vessel %s has been cleaned up." % updated_vessel
          vessels_on_node.remove(vessel)
      if not vessels_on_node:
        print "All vessels have been cleaned up."
        break
      else:
        print "%s vessels remain to be cleaned up." % len(vessels_on_node)
      print "Sleeping 10 seconds."
      time.sleep(10)

  finally:
    lockserver.destroy_lockserver_handle(lockserver_handle)


if __name__ == "__main__":
  if len(sys.argv) != 2:
    print 'Usage: ./stop_all_vessels_on_node.py "<node_id>"'
    sys.exit(1)
  maindb.init_maindb()
  stop_all_vessels_on_node(sys.argv[1])



"""
<Program Name>
  forms.py

<Started>
  October, 2008

<Author>
  Ivan Beschastnikh
  ivan@cs.washington.edu
  
  Jason Chen
  jchen@cs.washington.edu
<Purpose>

<Usage>
  For more information on forms in django see:
  http://docs.djangoproject.com/en/dev/topics/forms/
"""
from seattlegeni.website.control.models import GeniUser

from django.contrib.auth.forms import UserCreationForm as DjangoUserCreationForm
import django.forms as forms

from seattlegeni.common.exceptions import *
from seattlegeni.common.util import validations
from seattlegeni.website.control import interface

MAX_PUBKEY_UPLOAD_SIZE = 2048

class PubKeyField(forms.FileField):
  def clean(self,value,initial):
    forms.FileField.clean(self,value,initial)
    if value is None:
      return None
    if value.size > MAX_PUBKEY_UPLOAD_SIZE:
      raise forms.ValidationError, "Public key too large, file size limit is " + str(MAX_PUBKEY_UPLOAD_SIZE) + " bytes"
    # get the pubkey out of the uploaded file
    pubkey = value.read()
    try:
      validations.validate_pubkey_string(pubkey)
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return pubkey



class GeniUserCreationForm(DjangoUserCreationForm):
  affiliation = forms.CharField(error_messages={'required': 'Enter an Affiliation'})
  email = forms.CharField(label="E-mail Address", error_messages={'required': 'Enter an E-mail Address'})
  pubkey = PubKeyField(label="My Public Key", required=False)
  gen_upload_choice = forms.ChoiceField(label="", choices=((1, 'Generate key pairs for me'), (2, 'Let me upload my public key')))
  username = forms.CharField(label="Username", error_messages={'required': 'Enter a username'}, max_length=validations.USERNAME_MAX_LENGTH)
  
  def __init__(self, *args):
    DjangoUserCreationForm.__init__(self, *args)
    #self.fields['username'].error_messages['required'] = 'Enter a username'
    self.fields['password1'].error_messages['required'] = 'Enter a password'
    self.fields['password2'].error_messages['required'] = 'Verify your password'

  def clean_username(self):
    value = self.cleaned_data['username']
    try:
      validations.validate_username(value)
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return value
  
  def clean_password1(self):
    value = self.cleaned_data['password1']
    try:
      validations.validate_password(value)
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return value
  
  def clean_affiliation(self):
    value = self.cleaned_data['affiliation']
    try:
      validations.validate_affiliation(value)
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return value
  
  def clean_email(self):
    value = self.cleaned_data['email']
    try:
      validations.validate_email(value)
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return value





def gen_edit_user_form(field_list=None, *args, **kwargs):
  """
  <Purpose>
      Dynamically generates a EditUserForm depending on field_list.

  <Arguments>
      field_list:
          The profile view passes in the desired field that will be edited by the
          EditUserForm.

  <Exceptions>
      ValidationErrors raised by a incorrect field value.

  <Side Effects>
      None.

  <Returns>
      A EditUserForm object that is specific to the field list passed in.

  """
  class EditUserForm(forms.ModelForm):
    class Meta:
      model = GeniUser
      fields = field_list
      
    def __init__(self):
      super(EditUserForm, self).__init__(*args, **kwargs)
      
    def clean_affiliation(self):
      value = self.cleaned_data['affiliation']
      try:
        validations.validate_affiliation(value)
      except ValidationError, err:
        raise forms.ValidationError, str(err)
      return value
      
    def clean_email(self):
      value = self.cleaned_data['email']
      try:
        validations.validate_email(value)
      except ValidationError, err:
        raise forms.ValidationError, str(err)
      return value
    
  return EditUserForm()





class EditUserPasswordForm(forms.ModelForm):
  password1 = forms.CharField(label=("Password"), required=False, widget=forms.PasswordInput)
  password2 = forms.CharField(label=("Password confirmation"), required=False, widget=forms.PasswordInput, help_text = ("Enter the same password as above, for verification."))
  class Meta:
    model = GeniUser
    fields = ('password1','password2')
    
  def clean(self):
    data = self.cleaned_data
    if data['password1'] != data['password2']:
      raise forms.ValidationError(("The two password fields didn't match."))
    try:
      validations.validate_password(data['password1'])
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return data






class AutoRegisterForm(forms.ModelForm):
  username = forms.CharField(label="Username", error_messages={'required': 'Enter a username'}, max_length=validations.USERNAME_MAX_LENGTH)
  class Meta:
    model = GeniUser
    fields = ('username',)

  def clean_username(self):
    value = self.cleaned_data['username']
    try:
      validations.validate_username(value)
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return value
  
  '''    
  def clean(self):
    data = self.cleaned_data
    try:
      validations.validate_username(data['username'])
    except ValidationError, err:
      raise forms.ValidationError, str(err)
    return data    
  '''    





def gen_get_form(geni_user, req_post=None):
  """
  <Purpose>
      Dynamically generates a GetVesselsForm that has the right
      number vessels (the allowed number of vessels a user may
      acquire). Possibly generate a GetVesselsForm from an HTTP POST
      request.

  <Arguments>
      geni_user:
          geni_user object
      req_post:
          An HTTP POST request (django) object from which a
          GetVesselsForm may be instantiated. If this argument is
          not supplied, a blank form will be created

  <Exceptions>
      None.

  <Side Effects>
      None.

  <Returns>
      A GetVesselsForm object that is instantiated with a req_post
      (if given).
  """
      
  # the total number of vessels a user may acquire
  avail_vessel_credits = interface.get_available_vessel_credits(geni_user)
  
  # Dynamic generation of the options for numbers the user can request based
  # on their number of available vessel credits.
  if avail_vessel_credits == 0:
    step = [0]
  elif avail_vessel_credits < 10:
    step = range(1, avail_vessel_credits+1)
  elif avail_vessel_credits < 100:
    step = range(1, 10)
    step.extend(range(10, avail_vessel_credits+1,10))
  else:
    step = range(1, 10)
    step.extend(range(10, 101,10))
    step.extend(range(200, avail_vessel_credits+1, 100))
    
  if avail_vessel_credits not in step:
    step.append(avail_vessel_credits)

  # dynamically generate the get vessels form
  #get_vessel_choices = zip(range(1,max_num+1),range(1,max_num+1))
  get_vessel_choices = zip(step, step)
  
  # This is ugly (nested class definition, that is) and appears to have been
  # done as a way to avoid using a constructor but still make the value of
  # get_vessel_choices available to instances of this class.
  class GetVesselsForm(forms.Form):
    """
    <Purpose>
        Generates a form to acquire vessels by the user
    <Side Effects>
        None
    <Example Use>
        GetVesselsForm()
            to generate a blank form
        GetVesselsForm(post_request)
            to generate a form from an existing POST request
    """
    # maximum number of vessels a user is allowed to acquire
    #num = forms.ChoiceField(choices=get_vessel_choices, error_messages={'required' : 'Please enter the number of vessels to acquire'})
    num = forms.ChoiceField(choices=get_vessel_choices)
    
    # the various environment types the user may select from
    #env = forms.ChoiceField(choices=((1,'LAN'),(2,'WAN'),(3,'Random')), error_messages={'required' : 'Please enter the networking environment for vessels to acquire'})
    env = forms.ChoiceField(choices=(('wan','WAN'),('lan','LAN'),('nat','NAT'),('rand','Random')))
    
    def clean_num(self):
      value = int(self.cleaned_data['num'])
      if value < 1:
        raise forms.ValidationError("Invalid vessel number selection.")
      return value
    
    def clean_env(self):
      value = str(self.cleaned_data['env'])
      if not (value == 'wan' or value == 'lan' or value == 'nat' or value == 'rand'):
        raise forms.ValidationError("Invalid vessel type selection.")
      return value
    
    def get_errors_as_str(self):
      return str(self.errors)
  
  if req_post is None:
      return GetVesselsForm()
  return GetVesselsForm(req_post)



"""
<Program>
  views.py

<Started>
  October, 2008

<Author>
  Ivan Beschastnikh
  Jason Chen
  Justin Samuel

<Purpose>
  This module defines the functions that correspond to each possible request
  made through the html frontend. The urls.py file in this same directory
  lists the url paths which can be requested and the corresponding function name
  in this file which will be invoked.
"""

import os
import sys
import shutil
import subprocess
import xmlrpclib

# Needed to escape characters for the Android referrer...
import urllib

from django.http import HttpResponse
from django.http import HttpResponseRedirect
from django.contrib.auth.forms import AuthenticationForm
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse

#Used to display meaningful OpenID/OAuth error messages to the user
from django.contrib.messages.api import get_messages
from django.shortcuts import render_to_response, redirect
from social_auth.utils import setting
from django.template import RequestContext
# Any requests that change state on the server side (e.g. result in database
# modifications) need to be POST requests. This is for protecting against
# CSRF attacks (part, but not all, of that is our use of the CSRF middleware
# which only checks POST requests). Sometimes we use this decorator, sometimes
# we check the request type inside the view function.
from django.views.decorators.http import require_POST

from django.views.generic.simple import direct_to_template
from django.views.generic.simple import redirect_to

# Make available all of our own standard exceptions.
from seattlegeni.common.exceptions import *

# This is the logging decorator use use.
from seattlegeni.common.util.decorators import log_function_call
from seattlegeni.common.util.decorators import log_function_call_without_return

# For user registration input validation
from seattlegeni.common.util import validations

from seattlegeni.common.util import log

from seattlegeni.website import settings

# All of the work that needs to be done is passed through the controller interface.
from seattlegeni.website.control import interface

from seattlegeni.website.html import forms

from seattle.repyportability import *
add_dy_support(locals())

dy_import_module_symbols("rsa.r2py")






class LoggedInButFailedGetGeniUserError(Exception):
  """
  <Purpose>
  Indicates that a function tried to get a GeniUser record, and failed;
  while having passed the @login_required decorator. This means that a
  DjangoUser is logged in, but there is no corresponding GeniUser record.
  
  This exception should only be thrown from _validate_and_get_geniuser,
  and caught by methods with @login_required decorators.
  """





def _state_key_file_to_publickey_string(key_file_name):
  """
  Read a public key file from the the state keys directory and return it in
  a key string format.
  """
  fullpath = os.path.join(settings.SEATTLECLEARINGHOUSE_STATE_KEYS_DIR, key_file_name)
  return rsa_publickey_to_string(rsa_file_to_publickey(fullpath))





# The key used as the state key for new donations.
ACCEPTDONATIONS_STATE_PUBKEY = _state_key_file_to_publickey_string("acceptdonation.publickey")





def error(request):
  """
  <Purpose>
    If a OpenID/OAuth backend itself has an error(not a user or Seattle Clearinghouse's fault) 
    a user will get redirected here.  This can happen if the backend rejects the user or from
    user cancelation.

  <Arguments>
    request:
      An HTTP request object.

  <Exceptions>
    None

  <Side Effects>
    None

  <Returns>
    An HTTP response object that represents the error page.
  """
  #Retrieve information which caused an error 
  messages = get_messages(request)
  info =''
  try:
    user = _validate_and_get_geniuser(request)
    return profile(request, info, info, messages)
  except:
    return _show_login(request, 'accounts/login.html', {'messages' : messages}) 





@login_required
def associate_error(request):
  """
  <Purpose>
    If an error occured during the OpenId/OAuth association process a user will get
    redirected here.
  <Arguments>
    request:
      An HTTP request object.
    backend:
      An OpenID/OAuth backend ex google,facebook etc.
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    An HTTP response object that represents the associate_error page.
  """
  info=''
  error_msg = "Whoops, this account is already linked to another Seattle Clearninghouse user."
  return profile(request, info, error_msg)





def auto_register(request,backend=None,error_msgs=''):
  """
  <Purpose>
  Part of the SOCIAL_AUTH_PIPELINE whose order is mapped in settings.py.  If
  a user logs in with a OpenID/OAuth account and that account is not yet linked
  with a Clearinghouse account, he gets redirected here.
  If a valid username is entered then a new Seattle Clearinghouse user is created.

  <Arguments>
    request:
      An HTTP request object.
    backend:
      An OpenID/OAuth backend ex google,facebook etc.
  <Exceptions>

  <Side Effects>
    A new Seattle Clearinghouse user is created.
  <Returns>
    If a user passes in a valid username he continues the pipeline and moves
    forward in the auto register process.
  """
  # Check if a username is provided 
  username_form = forms.AutoRegisterForm()
  if request.method == 'POST' and request.POST.get('username'):
    name = setting('SOCIAL_AUTH_PARTIAL_PIPELINE_KEY', 'partial_pipeline')
    username_form = forms.AutoRegisterForm(request.POST)
    if username_form.is_valid():
      username = username_form.cleaned_data['username']
      try:
        interface.get_user_without_password(username)
        error_msgs ='That username is already in use.'
      except DoesNotExistError:
        request.session['saved_username'] = request.POST['username']
        backend = request.session[name]['backend']
        return redirect('socialauth_complete', backend=backend)  
  name = setting('SOCIAL_AUTH_PARTIAL_PIPELINE_KEY', 'partial_pipeline')
  backend=request.session[name]['backend']
  return render_to_response('accounts/auto_register.html', {'backend' : backend, 'error_msgs' : error_msgs, 'username_form' : username_form}, RequestContext(request))





@log_function_call_without_return
@login_required
def profile(request, info="", error_msg="", messages=""):
  """
  <Purpose>
    Display information about the user account.
    This method requires the request to represent a valid logged
    in user. See the top-level comment about the @login_required()
    decorator to achieve this property.  User account is editable through this 
    method.
  <Arguments>
    request:
      An HTTP request object.  
    info:
      Additional message to display at the top of the page in a green box.
    error_msg:
      Additional message to display at top of the page in a red box.
    messages
      Django social auth error message
  <Exceptions>
    None
  <Side Effects>
    None
  <Returns>
    An HTTP response object that represents the profile page.
  """
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  email_form = forms.gen_edit_user_form(instance=user)
  affiliation_form = forms.gen_edit_user_form(instance=user)
  password_form = forms.EditUserPasswordForm()
    
  if request.method == 'POST':
    if 'affiliation' in request.POST:
       affiliation_form = forms.gen_edit_user_form(('affiliation',), request.POST, instance=user)
       if affiliation_form.is_valid():
         new_affiliation = affiliation_form.cleaned_data['affiliation']
         interface.change_user_affiliation(user, new_affiliation)
         info ="Affiliation has been successfully changed to %s." % (user.affiliation)
    elif 'email' in request.POST:
       email_form = forms.gen_edit_user_form(('email',), request.POST, instance=user)
       if email_form.is_valid():
         new_email = email_form.cleaned_data['email']
         interface.change_user_email(user, new_email)
         info ="Email has been successfully changed to %s." % (user.email)
    elif 'password1' in request.POST:
       password_form = forms.EditUserPasswordForm( request.POST, instance=user)
       if password_form.is_valid():
         new_password = password_form.cleaned_data['password1']
         interface.change_user_password(user, new_password)
         info ="Password has been successfully changed"
  
  username = user.username
  affiliation = user.affiliation
  email = user.email
  port = user.usable_vessel_port
  has_privkey = user.user_privkey != None
  #currently not used, needed if editing user port is allowed
  #port_range = interface.get_useable_ports()
  #port_range_min = port_range[0]
  #port_range_max = port_range[-1]
  
  return direct_to_template(request, 'control/profile.html',
                            {'email_form' : email_form,
                             'affiliation_form' : affiliation_form,
                             'password_form' : password_form,
                             'username' : username,
                             'affiliation' : affiliation,
                             'email' : email,
                             'port' : port,
                             'api_key' : user.api_key,
                             'has_privkey' : has_privkey,
                             #'port_range_min' : port_range_min,
                             #'port_range_max' : port_range_max,
                             'info' : info,
                             'error_msg' : error_msg,
                             'messages' : messages})





def register(request):
  try:
    # check to see if a user is already logged in. if so, redirect them to profile.
    user = interface.get_logged_in_user(request)
  except DoesNotExistError:
    pass
  else:
    return HttpResponseRedirect(reverse("profile"))
  
  page_top_errors = []
  if request.method == 'POST':
    
    #TODO: what if the form data isn't in the POST request? we need to check for this.
    form = forms.GeniUserCreationForm(request.POST, request.FILES)
    # Calling the form's is_valid() function causes all form "clean_..." methods to be checked.
    # If this succeeds, then the form input data is validated per field-specific cleaning checks. (see forms.py)
    # However, we still need to do some checks which aren't doable from inside the form class.
    if form.is_valid():
      username = form.cleaned_data['username']
      password = form.cleaned_data['password1']
      affiliation = form.cleaned_data['affiliation']
      email = form.cleaned_data['email']
      pubkey = form.cleaned_data['pubkey']
      
      try:
        validations.validate_username_and_password_different(username, password)
      except ValidationError, err:
        page_top_errors.append(str(err))
      
      # NOTE: gen_upload_choice turns out to be a *string* when retrieved, hence '2'
      if form.cleaned_data['gen_upload_choice'] == '2' and pubkey == None:
        page_top_errors.append("Please select a public key to upload.")
      
      # only proceed with registration if there are no validation errors
      if page_top_errors == []:
        try:
          # we should never error here, since we've already finished validation at this point.
          # but, just to be safe...
          user = interface.register_user(username, password, email, affiliation, pubkey)
        except ValidationError, err:
          page_top_errors.append(str(err))
        else:
          return _show_login(request, 'accounts/login.html',
                             {'msg' : "Username %s has been successfully registered." % (user.username)})
  else:
    form = forms.GeniUserCreationForm()
  return direct_to_template(request, 'accounts/register.html', {'form' : form, 'page_top_errors' : page_top_errors })
  




def _show_login(request, ltemplate, template_dict, form=None):
    """
    <Purpose>
        Show the GENI login form

    <Arguments>
        request:
            An HTTP request object to use to populate the form
            
        ltemplate:
           The login template name to use for the login form. Right now
           this can be one of 'accounts/simplelogin.html' and
           'accounts/login.html'. They provide different ways of visualizing
           the login page.

        template_dict:
           The dictionary of arguments to pass to the template

        form:
           Either None or the AuthenticationForm to use as a 'form' argument
           to ltemplate. If form is None, a fresh AuthenticationForm() will be
           created and used.

    <Exceptions>
        None.

    <Side Effects>
        None. 

    <Returns>
        An HTTP response object that represents the login page on
        success.
    """
    if form == None:
        # initial page load
        form = AuthenticationForm()
        # set test cookie, but only once -- remove it on login
        #if not request.session.test_cookie_worked():
        request.session.set_test_cookie()
    template_dict['form'] = form
    return direct_to_template(request, ltemplate, template_dict)
  
  
  
  

def login(request):
  try:
    # check to see if a user is already logged in. if so, redirect them to profile.
    user = interface.get_logged_in_user(request)
  except DoesNotExistError:
    pass
  else:
    return HttpResponseRedirect(reverse("profile"))

  ltemplate = 'accounts/login.html'
  if request.method == 'POST':
    form = AuthenticationForm(request.POST)
    
    if not request.session.test_cookie_worked():
      request.session.set_test_cookie()
      return _show_login(request, ltemplate, {'err' : "Please enable your cookies and try again."}, form)

    if request.POST.has_key('jsenabled') and request.POST['jsenabled'] == 'false':
      return _show_login(request, ltemplate, {'err' : "Please enable javascript and try again."}, form)

    try:
      interface.login_user(request, request.POST['username'], request.POST['password'])
    except DoesNotExistError:
      return _show_login(request, ltemplate, {'err' : "Wrong username or password."}, form)
      
    # only clear out the cookie if we actually authenticate and login ok
    request.session.delete_test_cookie()
    
    return HttpResponseRedirect(reverse("profile"))
    
  # request type is GET, show a fresh login page
  return _show_login(request, ltemplate, {})





def logout(request):
  interface.logout_user(request)
  # TODO: We should redirect straight to login page
  return HttpResponseRedirect(reverse("profile"))





@login_required
def help(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  return direct_to_template(request, 'control/help.html', {'username': user.username})





def accounts_help(request):
  return direct_to_template(request, 'accounts/help.html', {})



@login_required
def mygeni(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  total_vessel_credits = interface.get_total_vessel_credits(user)
  num_acquired_vessels = len(interface.get_acquired_vessels(user))
  avail_vessel_credits = interface.get_available_vessel_credits(user)
  
  if num_acquired_vessels > total_vessel_credits:
    percent_total_used = 100
    over_vessel_credits = num_acquired_vessels - total_vessel_credits
  else:
    percent_total_used = int((num_acquired_vessels * 1.0 / total_vessel_credits * 1.0) * 100.0)
    over_vessel_credits = 0
  
  # total_vessel_credits, percent_total_used, avail_vessel_credits
  return direct_to_template(request, 'control/mygeni.html',
                            {'username' : user.username,
                             'total_vessel_credits' : total_vessel_credits,
                             'used_vessel_credits' : num_acquired_vessels,
                             'percent_total_used' : percent_total_used,
                             'avail_vessel_credits' : avail_vessel_credits,
                             'over_vessel_credits' : over_vessel_credits})





@login_required
def myvessels(request, get_form=False, action_summary="", action_detail="", remove_summary=""):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  # get_form of None means don't show the form to acquire vessels.
  if interface.get_available_vessel_credits(user) == 0:
    get_form = None
  elif get_form is False:
    get_form = forms.gen_get_form(user)

  # shared vessels that are used by others but which belong to this user (TODO)
  shvessels = []

  # this user's used vessels
  my_vessels_raw = interface.get_acquired_vessels(user)
  my_vessels = interface.get_vessel_infodict_list(my_vessels_raw)
  
  # this user's number of donations, max vessels, total vessels and free credits
  my_donations = interface.get_donations(user)
  my_max_vessels = interface.get_available_vessel_credits(user)	
  my_free_vessel_credits = interface.get_free_vessel_credits_amount(user)
  my_total_vessel_credits = interface.get_total_vessel_credits(user)

  for vessel in my_vessels:
    if vessel["expires_in_seconds"] <= 0:
      # We shouldn't ever get here, but just in case, let's handle it.
      vessel["expires_in"] = "Expired"
    else:
      days = vessel["expires_in_seconds"] / (3600 * 24)
      hours = vessel["expires_in_seconds"] / 3600 % 24
      minutes = vessel["expires_in_seconds"] / 60 % 60
      vessel["expires_in"] = "%dd %dh %dm" % (days, hours, minutes)

  # return the used resources page constructed from a template
  return direct_to_template(request, 'control/myvessels.html',
                            {'username' : user.username,
                             'num_vessels' : len(my_vessels),
                             'my_vessels' : my_vessels,
                             'sh_vessels' : shvessels,
                             'get_form' : get_form,
                             'action_summary' : action_summary,
                             'action_detail' : action_detail,
                             'my_donations' : len(my_donations),
                             'my_max_vessels' : my_max_vessels, 
                             'free_vessel_credits' : my_free_vessel_credits,
                             'total_vessel_credits' : my_total_vessel_credits,
                             'remove_summary' : remove_summary})





@login_required
def getdonations(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  domain = "https://" + request.get_host()
  
  return direct_to_template(request, 'control/getdonations.html',
                            {'username' : user.username,
                             'domain' : domain})





@login_required
def get_resources(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  # the request must be via POST. if not, bounce user back to My Vessels page
  if not request.method == 'POST':
    return myvessels(request)
  
  # try and grab form from POST. if it can't, bounce user back to My Vessels page
  get_form = forms.gen_get_form(user, request.POST)
  
  action_summary = ""
  action_detail = ""
  keep_get_form = False
  
  if get_form.is_valid():
    vessel_num = get_form.cleaned_data['num']
    vessel_type = get_form.cleaned_data['env']
    
    try:
      acquired_vessels = interface.acquire_vessels(user, vessel_num, vessel_type)
    except UnableToAcquireResourcesError, err:
      action_summary = "Unable to acquire vessels at this time."
      if str(err) == 'Acquiring NAT vessels is currently disabled. ':
        link = """<a href="https://seattle.cs.washington.edu/blog">blog</a>"""
        action_detail += str(err) + 'Please check our '+ link  +' to see when we have re-enabled NAT vessels.'
      else:
        action_detail += str(err)
      keep_get_form = True
    except InsufficientUserResourcesError:
      action_summary = "Unable to acquire vessels: you do not have enough vessel credits to fulfill this request."
      keep_get_form = True
  else:
    keep_get_form = True
  
  if keep_get_form == True:
    # return the original get_form, since the form wasn't valid (or there were errors)
    return myvessels(request, get_form, action_summary=action_summary, action_detail=action_detail)
  else:
    # return a My Vessels page with the updated vessels/vessel acquire details/errors
    return myvessels(request, False, action_summary=action_summary, action_detail=action_detail)
  
  
  
  

@login_required
def del_resource(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  # the request must be via POST. if not, bounce user back to My Vessels page
  if not request.method == 'POST':
    return myvessels(request)

  if not request.POST['handle']:
    return myvessels(request)
  
  # vessel_handle needs to be a list (even though we only add one handle), 
  # since get_vessel_list expects a list.
  vessel_handle = []
  vessel_handle.append(request.POST['handle'])
  remove_summary = ""
  
  try:
    # convert handle to vessel
    vessel_to_release = interface.get_vessel_list(vessel_handle)
  except DoesNotExistError:
    remove_summary = "Unable to remove vessel. The vessel you are trying to remove does not exist."
  except InvalidRequestError, err:
    remove_summary = "Unable to remove vessel. " + str(err)
  else:
    try:
      interface.release_vessels(user, vessel_to_release)
    except InvalidRequestError, err:
      remove_summary = "Unable to remove vessel. The vessel does not belong"
      remove_summary += " to you any more (maybe it expired?). " + str(err)
  
  return myvessels(request, remove_summary=remove_summary)






@login_required
def del_all_resources(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  # the request must be via POST. if not, bounce user back to My Vessels page
  if not request.method == 'POST':
    return myvessels(request)
  
  remove_summary = ""

  try:
    interface.release_all_vessels(user)
  except InvalidRequestError, err:
    remove_summary = "Unable to release all vessels: " + str(err)
  
  return myvessels(request, remove_summary=remove_summary)






@login_required
def renew_resource(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  # The request must be via POST. If not, bounce user back to My Vessels page.
  if not request.method == 'POST':
    return myvessels(request)
  
  if not request.POST.get('handle', ''):
    return myvessels(request)
  
  action_summary = ""
  action_detail = ""
  
  try:
    # Convert handle to vessel object.
    # Raises a DoesNotExistError if the vessel does not exist. This is more
    # likely to be an error than an expected case, so we let it bubble up.
    vessel_handle_list = [request.POST['handle']]
    vessel_to_renew = interface.get_vessel_list(vessel_handle_list)
  except DoesNotExistError:
    action_summary = "Unable to renew vessel: The vessel you are trying to delete does not exist."
  except InvalidRequestError, err:
    action_summary = "Unable to renew vessel."
    action_detail += str(err)
  else:
    try:
      interface.renew_vessels(user, vessel_to_renew)
    except InvalidRequestError, err:
      action_summary = "Unable to renew vessel: " + str(err)
    except InsufficientUserResourcesError, err:
      action_summary = "Unable to renew vessel: you are currently over your"
      action_summary += " vessel credit limit."
      action_detail += str(err)
  
  return myvessels(request, False, action_summary=action_summary, action_detail=action_detail)





@login_required
def renew_all_resources(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  # The request must be via POST. If not, bounce user back to My Vessels page.
  if not request.method == 'POST':
    return myvessels(request)
  
  action_summary = ""
  action_detail = ""
  
  try:
    interface.renew_all_vessels(user)
  except InvalidRequestError, err:
    action_summary = "Unable to renew vessels: " + str(err)
  except InsufficientUserResourcesError, err:
    action_summary = "Unable to renew vessels: you are currently over your"
    action_summary += " vessel credit limit."
    action_detail += str(err)
  
  return myvessels(request, False, action_summary=action_summary, action_detail=action_detail)





@login_required
def change_key(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  info = ""
  if request.method == 'GET':
    return direct_to_template(request, 'control/change_key.html',
                              {'username' : user.username,
                               'error_msg' : ""})

  # This is a POST, so figure out if a file was uploaded or if we are supposed
  # to generate a new key for the user.
  if request.POST.get('generate', False):
    interface.change_user_keys(user, pubkey=None)
    msg = "Your new keys have been generated. You should download them now."
    return profile(request, msg)
    
  else:
    file = request.FILES.get('pubkey', None)
    if file is None:
      msg = "You didn't select a public key file to upload." 
      return profile(request, info, msg)
      #return direct_to_template(request, 'control/change_key.html',
      #                          {'username' : user.username,
      #                           'error_msg' : msg})
    
    if file.size == 0 or file.size > forms.MAX_PUBKEY_UPLOAD_SIZE:
      msg = "Invalid file uploaded. The file size limit is " 
      msg += str(forms.MAX_PUBKEY_UPLOAD_SIZE) + " bytes."
      return profile(request, info, msg) 
      #direct_to_template(request, 'control/change_key.html',
      #                          {'username' : user.username,
      #                           'error_msg' : msg})
    
    pubkey = file.read()
    
    try:
      validations.validate_pubkey_string(pubkey)
    except ValidationError:
      msg = "Invalid public key uploaded."
      return profile(request, info, msg)
      #direct_to_template(request, 'control/change_key.html',
      #                          {'username' : user.username,
      #                           'error_msg' : msg})
    
    # If we made it here, the uploaded key is good.
    interface.change_user_keys(user, pubkey=pubkey)
    msg = "Your public key has been successfully changed."
    return profile(request, msg)





@login_required
def api_info(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  if request.method == 'GET':
    return direct_to_template(request, 'control/api_info.html',
                              {'username' : user.username,
                               'api_key' : user.api_key,
                               'msg' : ""})

  # This is a POST, so it should be generation of an API key.
  if not request.POST.get('generate_api_key', False):
    msg = "Sorry, we didn't understand your request."
    return profile(request, info, msg) 
    #direct_to_template(request, 'control/api_info.html',
    #                          {'username' : user.username,
    #                           'api_key' : user.api_key,
    #                           'msg' : msg})
    
  interface.regenerate_api_key(user)
  msg = "Your API key has been regenerated. Your old one will no longer work."
  msg += " You should update any places you are using the API key"
  msg += " (e.g. in programs using the XML-RPC client)."
  return profile(request,msg)
  #direct_to_template(request, 'control/api_info.html',
  #                         {'username' : user.username,
  #                           'api_key' : user.api_key,
  #                           'msg' : msg})





@log_function_call
@require_POST
@login_required
def del_priv(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  if user.user_privkey == "":
    msg = "Your private key has already been deleted."
  else:
    interface.delete_private_key(user)
    msg = "Your private key has been deleted."
  return profile(request, msg)





@log_function_call
@login_required
def priv_key(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  response = HttpResponse(user.user_privkey, mimetype='text/plain')
  response['Content-Disposition'] = 'attachment; filename=' + str(user.username) + '.privatekey'
  return response





@log_function_call
@login_required
def pub_key(request):
  try:
    user = _validate_and_get_geniuser(request)
  except LoggedInButFailedGetGeniUserError:
    return _show_failed_get_geniuser_page(request)
  
  response = HttpResponse(user.user_pubkey, mimetype='text/plain')
  response['Content-Disposition'] = 'attachment; filename=' + str(user.username) + '.publickey'
  return response





def download(request, username):
  validuser = True
  try:
    # validate that this username actually exists
    user = interface.get_user_for_installers(username)
  except DoesNotExistError:
    validuser = False

  templatedict = {}
  templatedict['username'] = username
  templatedict['validuser'] = validuser
  templatedict['domain'] = "https://" + request.get_host()
  # I need to build a URL for android to download the installer from.   (The
  # same installer is downloaded from the Google Play store for all users.) 
  # The URL is escaped twice (ask Akos why) and inserted in the referrer 
  # information in the URL.   
  #templatedict['android_installer_link'] = urllib.quote(urllib.quote(domain,safe=''),safe='')

  return direct_to_template(request, 'download/installers.html', templatedict)





def build_android_installer(request, username):
  """
  <Purpose>
    Allows the user to download a Android distribution of Seattle that will
    donate resources to user with 'username'.
  
  <Arguments>
    request:
      Django HttpRequest object
       
    username:
      A string representing the GENI user to which the installer will donate
      resources.
  
  <Exceptions>
    None
  
  <Side Effects>
    None
  
  <Returns>
    On failure, returns an HTTP response with a description of the error. On
    success, redirects the user to download the installer.
  """
  
  success, return_value = _build_installer(username, "android")
  
  if not success:
    error_response = return_value
    return error_response
  
  installer_url = return_value
  return HttpResponseRedirect(installer_url)





def build_win_installer(request, username):
  """
  <Purpose>
    Allows the user to download a Windows distribution of Seattle that will
    donate resources to user with 'username'.
  
  <Arguments>
    request:
      Django HttpRequest object
       
    username:
      A string representing the GENI user to which the installer will donate
      resources.
  
  <Exceptions>
    None
  
  <Side Effects>
    None
  
  <Returns>
    On failure, returns an HTTP response with a description of the error. On
    success, redirects the user to download the installer.
  """
  
  success, return_value = _build_installer(username, "windows")
  
  if not success:
    error_response = return_value
    return error_response
  
  installer_url = return_value
  return HttpResponseRedirect(installer_url)





def build_linux_installer(request, username):
  """
  <Purpose>
    Allows the user to download a Linux distribution of Seattle that will
    donate resources to user with 'username'.

  <Arguments>
    request:
      Django HttpRequest object

    username:
      A string representing the GENI user to which the installer will donate
      resources.

  <Exceptions>
    None

  <Side Effects>
    None

  <Returns>
    On failure, returns an HTTP response with a description of the error. On
    success, redirects the user to download the installer.
  """

  success, return_value = _build_installer(username, "linux")

  if not success:
    error_response = return_value
    return error_response

  installer_url = return_value
  return HttpResponseRedirect(installer_url)





def build_mac_installer(request, username):
  """
  <Purpose>
    Allows the user to download a Mac distribution of Seattle that will
    donate resources to user with 'username'.

  <Arguments>
    request:
      Django HttpRequest object

    username:
      A string representing the GENI user to which the installer will donate
      resources.

  <Exceptions>
    None

  <Side Effects>
    None

  <Returns>
    On failure, returns an HTTP response with a description of the error. On
    success, redirects the user to download the installer.
  """

  success, return_value = _build_installer(username, "mac")

  if not success:
    error_response = return_value
    return error_response

  installer_url = return_value
  return HttpResponseRedirect(installer_url)





def _build_installer(username, platform):
  """
  <Purpose>
    Builds an installer for the given platform that will donate resources to
    the user with the given username.
  
  <Arguments>
    username:
      A string representing the GENI user to which the installer will donate
      resources.
      
    platform:
      A string representing the platform for which to build the installer.
      Options include 'windows', 'linux', or 'mac'.
  
  <Exceptions>
    None
  
  <Side Effects>
    None
  
  <Returns>
    On success, returns (True, installer_url) where installer_url is URL from
    which the installer may be downloaded.
    
    On failure, returns (False, error_response) where error_repsponse is an
    HttpResponse which specifies what went wrong.
  """
  try:
    user = interface.get_user_for_installers(username)
    username = user.username
  except DoesNotExistError:
    error_response = HttpResponse("Couldn't get user.")
    return False, error_response

  try:
    xmlrpc_proxy = xmlrpclib.ServerProxy(settings.SEATTLECLEARINGHOUSE_INSTALLER_BUILDER_XMLRPC)
    
    vessel_list = [{'percentage': 80, 'owner': 'owner', 'users': ['user']}]
    
    user_data = {
      'owner': {'public_key': user.donor_pubkey},
      'user': {'public_key': ACCEPTDONATIONS_STATE_PUBKEY},
    }
    
    build_results = xmlrpc_proxy.build_installers(vessel_list, user_data)
  except:
    error_response = HttpResponse("Failed to build installer.")
    return False, error_response

  installer_url = build_results['installers'][platform]
  return True, installer_url





def donations_help(request, username):
  return direct_to_template(request, 'download/help.html', {'username' : username})





def _validate_and_get_geniuser(request):
  try:
    user = interface.get_logged_in_user(request)
  except DoesNotExistError:
    # Failed to get GeniUser record, but user is logged in
    raise LoggedInButFailedGetGeniUserError
  return user





@log_function_call_without_return
@login_required
def new_auto_register_user(request):
  msg = "Your account has been succesfully created. "
  msg += "If you would like to login without OpenID/OAuth please change your password now."
  return profile(request,msg)





def _show_failed_get_geniuser_page(request):
  err = "Sorry, we can't display the page you requested. "
  err += "If you are logged in as an administrator, you'll need to logout, and login with a Seattle Clearinghouse account. "
  err += "If you aren't logged in as an administrator, then this is a bug. Please contact us!"
  return _show_login(request, 'accounts/login.html', {'err' : err})



"""
<Program>
  test_renew_resources.py

<Started>
  Oct 13, 2009

<Author>
  Jason Chen <jchen@cs.washington.edu>
  Justin Samuel

<Purpose>
  Tests that the renew_resource view function 
  handles normal operation and exceptions correctly.
  
<Notes>
  See test_register.py for an explanation of our usage of the Django test client.
"""

# We import the testlib FIRST, as the test db settings 
# need to be set before we import anything else.
from seattlegeni.tests import testlib

from seattlegeni.common.exceptions import *
from seattlegeni.website.control import interface
from seattlegeni.website.control import models

from django.contrib.auth.models import User as DjangoUser

# The django test client emulates a webclient, and returns what django
# considers the final rendered result (in HTML). This allows to test purely the view
# functions. 
from django.test.client import Client


# Declare our mock functions
def mock_get_logged_in_user(request):
  geniuser = models.GeniUser(username='tester', password='password', email='test@test.com',
                             affiliation='test affil', user_pubkey='user_pubkey',
                             user_privkey='user_privkey', donor_pubkey='donor_pubkey',
                             usable_vessel_port='12345', free_vessel_credits=10)
  return geniuser

def mock_get_vessel_list(vesselhandle_list):
  return ['test', 'test2']

def mock_get_vessel_list_throws_DoesNotExistError(vesselhandle_list):
  raise DoesNotExistError

def mock_get_vessel_list_throws_InvalidRequestError(vesselhandle_list):
  raise InvalidRequestError

def mock_renew_vessels(geniuser, vessel_list):
  pass

def mock_renew_vessels_throws_InvalidRequestError(geniuser, vessel_list):
  raise InvalidRequestError

c = Client()
good_data = {'handle':'12345'}
interface.get_vessel_list = mock_get_vessel_list

def main():
  
  # Setup test environment
  testlib.setup_test_environment()
  testlib.setup_test_db()
  
  try:
    login_test_user()
    
    test_normal()
    test_get_vessel_list_throws_DoesNotExistError()
    test_get_vessel_list_throws_InvalidRequestError()
    test_renew_vessels_throws_InvalidRequestError()
    
    print "All tests passed."
    
  finally:
    testlib.teardown_test_db()
    testlib.teardown_test_environment()



def test_normal():
  """
  <Purpose>
    Tests normal behavior.
  """
  interface.get_vessel_list = mock_get_vessel_list
  interface.renew_vessels = mock_renew_vessels
  response = c.post('/html/renew_resource', good_data, follow=True)
  
  assert(response.status_code == 200)
  assert(response.template[0].name == 'control/myvessels.html')



def test_get_vessel_list_throws_DoesNotExistError():
  """
  <Purpose>
    Tests behavior if interface.get_vessel_list throws a DoesNotExistError
  """
  interface.get_vessel_list = mock_get_vessel_list_throws_DoesNotExistError
  interface.renew_vessels = mock_renew_vessels
  response = c.post('/html/renew_resource', good_data, follow=True)
  
  assert(response.status_code == 200)
  assert("Unable to renew" in response.content)
  assert(response.template[0].name == 'control/myvessels.html')



def test_get_vessel_list_throws_InvalidRequestError():
  """
  <Purpose>
    Tests behavior if interface.get_vessel_list throws an InvalidRequestError 
  """
  interface.get_vessel_list = mock_get_vessel_list_throws_InvalidRequestError
  interface.renew_vessels = mock_renew_vessels
  response = c.post('/html/renew_resource', good_data, follow=True)
  
  assert(response.status_code == 200)
  assert("Unable to renew" in response.content)
  assert(response.template[0].name == 'control/myvessels.html')



def test_renew_vessels_throws_InvalidRequestError():
  """
  <Purpose>
    Tests behavior if interface.renew_vessels throws an InvalidRequestError
  """
  interface.get_vessel_list = mock_get_vessel_list
  interface.renew_vessels = mock_renew_vessels_throws_InvalidRequestError
  response = c.post('/html/renew_resource', good_data, follow=True)
  
  assert(response.status_code == 200)
  assert("Unable to renew" in response.content)
  assert(response.template[0].name == 'control/myvessels.html')
  
  
  
# Creates a test user in the test db, and uses the test client to 'login',
# so all views that expect @login_required will now pass the login check. 
def login_test_user():
  # uses the mock get_logged_in_user function that represents a logged in user
  interface.get_logged_in_user = mock_get_logged_in_user
  
  user = DjangoUser.objects.create_user('tester', 'test@test.com', 'testpassword')
  user.save()
  c.login(username='tester', password='testpassword')



if __name__=="__main__":
  main()



"""
<Program>
  log.py

<Started>
  6 July 2009

<Author>
  Justin Samuel

<Purpose>
  This module provides logging functionality to be used within seattlegeni.
  All seattlegeni code should log through this module rather than using
  print() or directly using the repy logging module. This ensures that only
  one module (this one) determines where log messages actually go for all
  of seattlegeni. (This module may in turn use the repy logging module, but
  client code should be unaware of that.)
  
  Logging output done by this module will try to include a request_id with
  each log message so that it is possible to differentiate multiple requests
  or threads running at the same time.
  
  Currently, our custom django middleware at website/middleware/logrequest.py
  calls log_start_request() when a new request comes in.
  
  Using this module from outside of a django website means that client code
  should first call log_start_request() or set_request_id() if it wants
  log messages from a thread to be logged with an identifier.
"""

import random
import sys
import threading
from datetime import datetime




# We use a thread-local data store for tracking the request identifier
# so that we can log the associated request id even when the request
# object isn't among a function's arguments.
request_context = threading.local()





LOG_LEVEL_DEBUG = 1
LOG_LEVEL_INFO = 2
LOG_LEVEL_ERROR = 3
LOG_LEVEL_CRITICAL = 4
LOG_LEVEL_NONE = 5

# Default log level of DEBUG.
loglevel = LOG_LEVEL_DEBUG





def set_log_level(level):
  global loglevel
  loglevel = level





def debug(message):
  if loglevel <= LOG_LEVEL_DEBUG:
    print >> sys.stderr, _get_time() + " DEBUG " + _get_request_id() + " " + str(message)
    sys.stderr.flush()





def info(message):
  if loglevel <= LOG_LEVEL_INFO:
    print >> sys.stderr, _get_time() + " INFO " + _get_request_id() + " " + str(message)
    sys.stderr.flush()





def error(message):
  if loglevel <= LOG_LEVEL_ERROR:
    print >> sys.stderr, _get_time() + " ERROR " + _get_request_id() + " " + str(message)
    sys.stderr.flush()





def critical(message):
  if loglevel <= LOG_LEVEL_CRITICAL:
    print >> sys.stderr, _get_time() + " CRITICAL " + _get_request_id() + " " + str(message)
    sys.stderr.flush()





def _get_request_id():
  if not hasattr(request_context, 'request_id'):
    set_request_id()
  return request_context.request_id





def _get_time():
  return "[" + datetime.now().isoformat(' ') + "]"





def log_start_request(request):
  """
  Create a unique request id and store it in request_context if it isn't
  already set there. This way it will be available to future log calls.
  """
  request_id = _generate_request_id_from_request(request)
  set_request_id(request_id)

  messagelist = []

  messagelist.append("Request started.")
  
  # See http://docs.djangoproject.com/en/dev/ref/request-response/ for a list
  # of attributes of the HttpRequest object that we could potentially log.
  messagelist.append(request.method)
  messagelist.append(request.path)
  
  # Log the request variables.
  # We ignore the fact that there could be query string variables in a POST request.
  if request.method == "GET":
    messagelist.append(str(request.GET))
  else:
    clean_post_querydict = request.POST.copy()
    for key in request.POST:
      # TODO: need to prevent logging password/apikey in xmlrpc requests
      if "password" in key:
        clean_post_querydict[key] = "[REMOVED]"
    messagelist.append(str(clean_post_querydict))
  
  if request.user.is_authenticated():
    messagelist.append("logged_in_user:" + str(request.user.username))
  else:
    messagelist.append("logged_in_user:-")
  
  info(' '.join(messagelist))





def set_request_id(request_id=None):
  """
  Sets the request id. This will be used in all future log messages performed
  by the current thread.
  """
  if request_id is None:
    request_id = _generate_request_id()
    
  request_context.request_id = str(request_id)





def _generate_request_id():
  RANDMIN = 100000000
  RANDMAX = 999999999
  return str(random.randint(RANDMIN, RANDMAX))





def _generate_request_id_from_request(request):
  """
  Create a unique tag for the request, to make it easier to follow its log
  entries.
  
  Based on the idea from:
  http://www.fairviewcomputing.com/blog/2008/03/05/django-request-logging/
  """
  # If Apache's mod_unique_id is in use, use that. Otherwise, create one.
  if request.META.has_key('UNIQUE_ID'):
    return str(request.META['UNIQUE_ID'])
  else:
    return _generate_request_id()




"""
<Program>
  logrequest.py

<Started>
  6 July 2009

<Author>
  Justin Samuel

<Purpose>
  We register "middleware" with django to help of log requests, including being
  able to uniquely identify all future logging done as part of the same request.

  Documenation on django middleware: http://www.djangobook.com/en/1.0/chapter15/
"""


from seattlegeni.common.util import log

class LogRequestMiddleware(object):
  
  def process_request(self, request):
    # Leave the actual logging work to code in the log module.
    log.log_start_request(request)
    
    # Returning None indicates that the request should continue to be processed.
    return None



#pragma out
#pragma error OK

# The seattlegeni testlib must be imported first.
from seattlegeni.tests import testlib

from seattlegeni.tests import mocklib

from seattlegeni.common.api import maindb

from seattlegeni.common.exceptions import *

from seattlegeni.common.util import validations

from seattlegeni.website.control import interface

from seattlegeni.website.tests import testutil

import unittest



mocklib.mock_lockserver_calls()





class SeattleGeniTestCase(unittest.TestCase):


  def setUp(self):
    # Setup a fresh database for each test.
    testlib.setup_test_db()



  def tearDown(self):
    # Cleanup the test database.
    testlib.teardown_test_db()



  def test_regenerate_user_key(self):
    
    pubkey = "1 2"
    privkey = "3 4 5"
    donor_key = "6 7"
    
    # Create a user who will be doing the acquiring.
    user = maindb.create_user("testuser", "password", "example@example.com", "affiliation", 
                              pubkey, privkey, donor_key)
    userport = user.usable_vessel_port

    vesselcount = 4
    
    # Have every vessel acquisition to the backend request succeed.
    calls_results = [True] * vesselcount
    mocklib.mock_backend_acquire_vessel(calls_results)
    
    testutil.create_nodes_on_different_subnets(vesselcount, [userport])

    # Acquire all vessels on behalf of this user.
    all_vessels_list = interface.acquire_vessels(user, vesselcount, 'rand')

    # Release 2 vessels.
    released_vessels_list = all_vessels_list[:2]
    kept_vessels_list = all_vessels_list[2:]
    interface.release_vessels(user, released_vessels_list)
    
    # Ensure all of the vessels are marked as having user keys in sync.
    for vessel in all_vessels_list:
      # Get a fresh vessel from the db.
      vessel = maindb.get_vessel(vessel.node.node_identifier, vessel.name)
      self.assertTrue(vessel.user_keys_in_sync)

    # We expect a single key to be generated through the keygen api (the new
    # user public key).
    mocklib.mock_keygen_generate_keypair([("55 66", "77 88 99")])
    
    interface.change_user_keys(user, pubkey=None)
    
    # Get a new user object from the database.
    user = maindb.get_user(user.username)
    
    # Make sure the user's key changed.
    self.assertEqual(user.user_pubkey, "55 66")
    self.assertEqual(user.user_privkey, "77 88 99")
    
    # Make sure that all of the vessels the user has access to (and no other
    # vessels) are marked as needing user keys to be sync'd.
    # Ensure all of the vessels are marked as having user keys in sync.
    for vessel in kept_vessels_list:
      # Get a fresh vessel from the db.
      vessel = maindb.get_vessel(vessel.node.node_identifier, vessel.name)
      self.assertFalse(vessel.user_keys_in_sync)

    for vessel in released_vessels_list:
      # Get a fresh vessel from the db.
      vessel = maindb.get_vessel(vessel.node.node_identifier, vessel.name)
      self.assertTrue(vessel.user_keys_in_sync)





  def test_set_user_key(self):
    
    pubkey = "1 2"
    privkey = "3 4 5"
    donor_key = "6 7"
    
    # Create a user who will be doing the acquiring.
    user = maindb.create_user("testuser", "password", "example@example.com", "affiliation", 
                              pubkey, privkey, donor_key)
    userport = user.usable_vessel_port

    vesselcount = 4
    
    # Have every vessel acquisition to the backend request succeed.
    calls_results = [True] * vesselcount
    mocklib.mock_backend_acquire_vessel(calls_results)
    
    testutil.create_nodes_on_different_subnets(vesselcount, [userport])

    # Acquire all vessels on behalf of this user.
    all_vessels_list = interface.acquire_vessels(user, vesselcount, 'rand')

    # Release 2 vessels.
    released_vessels_list = all_vessels_list[:2]
    kept_vessels_list = all_vessels_list[2:]
    interface.release_vessels(user, released_vessels_list)
    
    # Ensure all of the vessels are marked as having user keys in sync.
    for vessel in all_vessels_list:
      # Get a fresh vessel from the db.
      vessel = maindb.get_vessel(vessel.node.node_identifier, vessel.name)
      self.assertTrue(vessel.user_keys_in_sync)

    # We expect no keys to be generated through the keygen api.
    mocklib.mock_keygen_generate_keypair([])
    
    interface.change_user_keys(user, pubkey="55 66")
    
    # Get a new user object from the database.
    user = maindb.get_user(user.username)
    
    # Make sure the user's key changed.
    self.assertEqual(user.user_pubkey, "55 66")
    self.assertEqual(user.user_privkey, None)
    
    # Make sure that all of the vessels the user has access to (and no other
    # vessels) are marked as needing user keys to be sync'd.
    # Ensure all of the vessels are marked as having user keys in sync.
    for vessel in kept_vessels_list:
      # Get a fresh vessel from the db.
      vessel = maindb.get_vessel(vessel.node.node_identifier, vessel.name)
      self.assertFalse(vessel.user_keys_in_sync)

    for vessel in released_vessels_list:
      # Get a fresh vessel from the db.
      vessel = maindb.get_vessel(vessel.node.node_identifier, vessel.name)
      self.assertTrue(vessel.user_keys_in_sync)





  def test_set_invalid_user_key(self):
    
    pubkey = "1 2"
    privkey = "3 4 5"
    donor_key = "6 7"
    
    # Create a user who will be doing the acquiring.
    user = maindb.create_user("testuser", "password", "example@example.com", "affiliation", 
                              pubkey, privkey, donor_key)
    
    func = interface.change_user_keys
    args = (user, "abc")
    self.assertRaises(ValidationError, func, *args)





def run_test():
  unittest.main()



if __name__ == "__main__":
  run_test()



"""
<Program>
  seattleclearinghouse_xmlrpc.py

<Started>
  6/28/2009

<Author>
  Jason Chen
  Justin Samuel

<Purpose>
  A client library for communicating with the SeattleClearinghouse XMLRPC Server.
  
  Your Python scripts can import this library, create an instance of the
  SeattleClearinghouseClient class, then call methods on the object to perform XMLRPC
  calls through the SeattleClearinghouse XMLRPC API.

  Full tutorials on using this library, see:
  https://seattle.cs.washington.edu/wiki/SeattleGeniClientLib
  
  In order to perform secure SSL communication with SeattleClearinghouse:
    * You must have M2Crypto installed.
    * You must set the value of CA_CERTIFICATES_FILE to the location of a PEM
      file containing CA certificates that you trust. If you don't know where
      this is on your own system, you can download this file from a site you
      trust. One such place to download this file from is:
        http://curl.haxx.se/ca/cacert.pem
        
  If you can't fulfill the above requirements, you can still use this client with
  XMLRPC servers that use https but you will be vulnerable to a man-in-the-middle
  attack. To enable this insecure mode, include the argument:
    allow_ssl_insecure=True
  when creating a SeattleClearinghouseClient instance.
  
<Notes>
  All methods of the client class may raise the following errors in addition to
  any others described in the method's docstring:
    CommunicationError
    AuthenticationError
    InvalidRequestError
    InternalError
  The safest way to be certain to catch any of these errors  is to the catch
  their base class:
    SeattleClearinghouseError
"""

import os
import socket
import xmlrpclib

# If a user does not provide us with an API key, we'll need to load
# their private key instead.
from repyportability import add_dy_support
add_dy_support(locals())

# Location of a file containing one or more PEM-encoded CA certificates
# concatenated together. This is required if using allow_ssl_insecure=False.
# By default it looks for a cacert.pem file in the same directory as this
# python module is in.
DEFAULT_CA_CERTIFICATES_FILE = os.path.join(os.path.dirname(__file__), "cacert.pem")

# The location of the SeattleClearinghouse XMLRPC server to use.
DEFAULT_XMLRPC_URL = "https://seattleclearinghouse.poly.edu/xmlrpc/"

# SeattleClearinghouse XMLRPC Fault Code Constants
FAULTCODE_INTERNALERROR = 100
FAULTCODE_AUTHERROR = 101
FAULTCODE_INVALIDREQUEST = 102
FAULTCODE_NOTENOUGHCREDITS = 103
FAULTCODE_UNABLETOACQUIRE = 105



class SeattleClearinghouseClient(object):
  """
  Implementation of an XMLRPC client for communicating with a SeattleClearinghouse
  server. This uses the public API described at:
  https://seattle.cs.washington.edu/wiki/SeattleGeniApi
  """
  
  def __init__(self, username, api_key=None, private_key_string=None,
               xmlrpc_url=None,
               allow_ssl_insecure=None,
               ca_certs_file=None):
    
    if xmlrpc_url is None:
      xmlrpc_url = DEFAULT_XMLRPC_URL
      
    if allow_ssl_insecure is None:
      allow_ssl_insecure = False
      
    if ca_certs_file is None:
      ca_certs_file = DEFAULT_CA_CERTIFICATES_FILE
    
    if not isinstance(username, basestring):
      raise TypeError("username must be a string")
    
    if api_key is not None:
      if not isinstance(api_key, basestring):
        raise TypeError("api_key must be a string")
    else:
      if not private_key_string:
        raise TypeError("private_key_string must be provided if api_key is not")
      if not isinstance(private_key_string, basestring):
        raise TypeError("private_key_string must be a string")
      
    if not isinstance(xmlrpc_url, basestring):
      raise TypeError("xmlrpc_url must be a string")
    if not isinstance(allow_ssl_insecure, bool):
      raise TypeError("allow_ssl_insecure must be True or False")
    if not isinstance(ca_certs_file, basestring):
      raise TypeError("ca_certs_file must be a string")
    
    if allow_ssl_insecure:
      self.proxy = xmlrpclib.Server(xmlrpc_url)
    else:
      ssl_transport = _get_ssl_transport(ca_certs_file)
      self.proxy = xmlrpclib.Server(xmlrpc_url, transport=ssl_transport)

    if not api_key:
      api_key = self._get_api_key(username, private_key_string)
    
    self.auth = {'username':username, 'api_key':api_key}



  def _get_api_key(self, username, private_key_string):
    # Normally we try not to import modules anywhere but globally,
    # but I'd like to keep this xmlrpc client usable without repy files
    # available when the user provides their api key and doesn't require
    # it to be retrieved.
    try:
      dy_import_module_symbols("rsa.r2py")
    except ImportError, e:
      raise SeattleClearinghouseError("Unable to get API key from SeattleClearinghouse " +
                             "because a required python or repy module " + 
                             "cannot be found:" + str(e))
    
    # This will raise a ValueError if the private key is not valid.
    private_key_dict = rsa_string_to_privatekey(private_key_string)
    
    encrypted_data = self.proxy.get_encrypted_api_key(username)
    decrypted_data = rsa_decrypt(encrypted_data, private_key_dict)
    split_data = decrypted_data.split("!")

    # The encrypted data has 20 bytes of random data followed by a "!" which
    # is then followed by the actual API key. If the private key was the wrong
    # key, we will end up with garbage data (if it was an invalid key, it
    # might be empty, though). So, we're going to make the fairly safe
    # assumption that the odds of a random decryption with the wrong key
    # resulting in data that starts with 20 bytes which aren't exclamation
    # marks followed by a single exclamation mark and no others is pretty low.
    if len(split_data) != 2 or len(split_data[0]) != 20:
      raise AuthenticationError("The provided private key does not appear " +
                                "to correspond to this account's public key: " +
                                "encrypted API key could not be decrypted.")
    api_key = split_data[1]
    
    return api_key


  
  def _do_call(self, function, *args):
    try:
      return function(self.auth, *args)
    except socket.error, err:
      raise CommunicationError("XMLRPC failed: " + str(err))
    except xmlrpclib.Fault, fault:
      if fault.faultCode == FAULTCODE_AUTHERROR:
        raise AuthenticationError
      elif fault.faultCode == FAULTCODE_INVALIDREQUEST:
        raise InvalidRequestError(fault.faultString)
      elif fault.faultCode == FAULTCODE_NOTENOUGHCREDITS:
        raise NotEnoughCreditsError(fault.faultString)
      elif fault.faultCode == FAULTCODE_UNABLETOACQUIRE:
        raise UnableToAcquireResourcesError(fault.faultString)
      else:
        raise InternalError(fault.faultString)



  def _do_pwauth_call(self, function, password, *args):
    """For use by calls that require a password rather than an api key."""
    pwauth = {'username':self.auth['username'], 'password':password}
    try:
      return function(pwauth, *args)
    except socket.error, err:
      raise CommunicationError("XMLRPC failed: " + str(err))
    except xmlrpclib.Fault, fault:
      if fault.faultCode == FAULTCODE_AUTHERROR:
        raise AuthenticationError
      elif fault.faultCode == FAULTCODE_INVALIDREQUEST:
        raise InvalidRequestError(fault.faultString)
      elif fault.faultCode == FAULTCODE_NOTENOUGHCREDITS:
        raise NotEnoughCreditsError(fault.faultString)
      elif fault.faultCode == FAULTCODE_UNABLETOACQUIRE:
        raise UnableToAcquireResourcesError(fault.faultString)
      else:
        raise InternalError(fault.faultString)



  def acquire_lan_resources(self, count):
    """
    <Purpose>
      Acquire LAN vessels.
    <Arguments>
      count
        The number of vessels to acquire.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
      SeattleClearinghouseNotEnoughCredits
        If the account does not have enough available vessel credits to fulfill
        the request.
    <Side Effects>
      If successful, 'count' LAN vessels have been acquired for the account.
    <Returns>
      A list of vessel handles of the acquired vessels.
    """
    return self.acquire_resources('lan', count)



  def acquire_wan_resources(self, count):
    """
    <Purpose>
      Acquire WAN vessels.
    <Arguments>
      count
        The number of vessels to acquire.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
      SeattleClearinghouseNotEnoughCredits
        If the account does not have enough available vessel credits to fulfill
        the request.
    <Side Effects>
      If successful, 'count' WAN vessels have been acquired for the account.
    <Returns>
      A list of vessel handles of the acquired vessels.
    """
    return self.acquire_resources('wan', count)



  def acquire_nat_resources(self, count):
    """
    <Purpose>
      Acquire NAT vessels.
    <Arguments>
      count
        The number of vessels to acquire.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
      SeattleClearinghouseNotEnoughCredits
        If the account does not have enough available vessel credits to fulfill
        the request.
    <Side Effects>
      If successful, 'count' NAT vessels have been acquired for the account.
    <Returns>
      A list of vessel handles of the acquired vessels.
    """
    return self.acquire_resources('nat', count)



  def acquire_random_resources(self, count):
    """
    <Purpose>
      Acquire vessels (they can be LAN, WAN, NAT, or any combination of these).
    <Arguments>
      count
        The number of vessels to acquire.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
      SeattleClearinghouseNotEnoughCredits
        If the account does not have enough available vessel credits to fulfill
        the request.
    <Side Effects>
      If successful, 'count' vessels have been acquired for the account.
    <Returns>
      A list of vessel handles of the acquired vessels.
    """
    return self.acquire_resources('random', count)
    
    
    
  def acquire_resources(self, res_type, count):
    """
    <Purpose>
      Acquire vessels.
    <Arguments>
      res_type
        A string describing the type of vessels to acquire.
      count
        The number of vessels to acquire.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
      SeattleClearinghouseNotEnoughCredits
        If the account does not have enough available vessel credits to fulfill
        the request.
    <Side Effects>
      If successful, 'count' vessels have been acquired for the account.
    <Returns>
      A list of vessel handles of the acquired vessels.
    """
    if not isinstance(res_type, basestring):
      raise TypeError("res_type must be a string")
    if type(count) not in [int, long]:
      raise TypeError("count must be an integer")
    
    rspec = {'rspec_type':res_type, 'number_of_nodes':count}
    return self._do_call(self.proxy.acquire_resources, rspec)



  def acquire_specific_vessels(self, handlelist):
    """
    <Purpose>
      Attempt to acquire specific vessels.
    <Arguments>
      handlelist
        A list of vessel handles.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
      SeattleClearinghouseNotEnoughCredits
        If the account does not have enough available vessel credits to fulfill
        the request.
    <Side Effects>
      If successful, zero or more vessels from handlelist have been acquired.
    <Returns>
      A list of vessel handles of the acquired vessels.
    """
    _validate_handle_list(handlelist)
    return self._do_call(self.proxy.acquire_specific_vessels, handlelist)



  def release_resources(self, handlelist):
    """
    <Purpose>
      Release vessels.
    <Arguments>
      handlelist
        A list of handles as returned by acquire_vessels() or found in the
        'handle' key of the dictionaries returned by get_resource_info().
    <Exceptions>
      The common exceptions described in the module comments.
    <Side Effects>
      If successful, the vessels in handlelist have been released. If not
      successful, it is possible that a partial set of the vessels was
      released.
    <Returns>
      None
    """
    _validate_handle_list(handlelist)
    return self._do_call(self.proxy.release_resources, handlelist)

      
      
  def renew_resources(self, handlelist):
    """
    <Purpose>
      Renew vessels.
    <Arguments>
      handlelist
        A list of handles as returned by acquire_vessels() or found in the
        'handle' key of the dictionaries returned by get_resource_info().
    <Exceptions>
      The common exceptions described in the module comments, as well as:
      SeattleClearinghouseNotEnoughCredits
        If the account is currently over its vessel credit limit, then vessels
        cannot be renewed until the account is no longer over its credit limit.
    <Side Effects>
      If successful, the vessels in handlelist have been renewed. If not
      successful, it is possible that a partial set of the vessels was
      renewed.
    <Returns>
      None
    """
    _validate_handle_list(handlelist)
    return self._do_call(self.proxy.renew_resources, handlelist)
    


  def get_resource_info(self):
    """
    <Purpose>
      Obtain information about acquired vessels.
    <Arguments>
      None
    <Exceptions>
      The common exceptions described in the module comments, as well as:
    <Side Effects>
      None
    <Returns>
      A list of dictionaries, where each dictionary describes a vessel that
      is currently acquired by the account.
    """
    return self._do_call(self.proxy.get_resource_info)
      
      
      
  def get_account_info(self):
    """
    <Purpose>
      Obtain information about the account.
    <Arguments>
      None
    <Exceptions>
      The common exceptions described in the module comments, as well as:
    <Side Effects>
      None
    <Returns>
      A dictionary with information about the account.
    """
    return self._do_call(self.proxy.get_account_info)
    
    
    
  def get_public_key(self):
    """
    <Purpose>
      Obtain the public key of the account.
    <Arguments>
      None
    <Exceptions>
      The common exceptions described in the module comments, as well as:
        None
    <Side Effects>
      None
    <Returns>
      A string containing the public key of the account.
    """
    return self._do_call(self.proxy.get_public_key)



  def set_public_key(self, password, pubkeystring):
    """
    <Purpose>
      Set the public key of the account.
    <Arguments>
      password
        The account password. This is required because changing the public
        key of the account cannot be done with just the api key.
      pubkeystring
        A string representing the new public key to be set for the account.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
        InvalidRequestError
          If the pubkey is invalid.
    <Side Effects>
      The public key of the account is changed and will be updated on all
      vessels the account has acquired.
    <Returns>
      None
    """
    self._do_pwauth_call(self.proxy.set_public_key, password, pubkeystring)



  def regenerate_api_key(self, password):
    """
    <Purpose>
      Generate a new API key for the account..
    <Arguments>
      password
        The account password. This is required because changing the api
        key of the account cannot be done with just the current api key.
    <Exceptions>
      The common exceptions described in the module comments, as well as:
        None
    <Side Effects>
      The account's api key has been changed.
    <Returns>
      The new api key for the account.
    """
    api_key = self._do_pwauth_call(self.proxy.regenerate_api_key, password)
    self.auth['api_key'] = api_key
    return api_key
   
      


def _validate_handle_list(handlelist):
  """
  Raise a TypeError or ValueError if handlelist is not a non-empty list of
  string.
  """
  if not isinstance(handlelist, list):
    raise TypeError("Invalid data type for handle list: " + 
                    str(type(handlelist)))
  
  for handle in handlelist:
    if not isinstance(handle, basestring):
      raise TypeError("Invalid data type for a handle in the handle list: " + 
                      str(type(handle)))
  
  if not handlelist:
    raise ValueError("Given handlelist is empty.")





def _get_ssl_transport(ca_certs_file):
  """
  Returns an object usable as the transport for an xmlrpclib proxy. This will
  be an M2Crypto.m2xmlrpclib.SSL_Transport that has been configured with a
  context that has the ca_certs_file loaded, will not allow SSLv2, and will
  reject certificate names that don't match the hostname.
  """
  try:
    import M2Crypto
  except ImportError, err:
    raise ImportError("In order to use the SeattleClearinghouse XMLRPC client with " + 
                      "allow_ssl_insecure=False, you need M2Crypto " + 
                      "installed. " + str(err))
  
  # We don't define this class until here because otherwise M2Crypto may not
  # be available.
  class M2CryptoSSLTransport(M2Crypto.m2xmlrpclib.SSL_Transport):
    def request(self, host, handler, request_body, verbose=0):
      if host.find(":") == -1:
        host = host + ":443"
      return M2Crypto.m2xmlrpclib.SSL_Transport.request(self, host, handler,
                                                        request_body, verbose)

  ctx = M2Crypto.SSL.Context("sslv3")
  ctx.set_verify(M2Crypto.SSL.verify_peer | 
                 M2Crypto.SSL.verify_fail_if_no_peer_cert, depth=9)
  if ctx.load_verify_locations(ca_certs_file) != 1:
    raise SeattleClearinghouseError("No CA certs found in file: " + ca_certs_file)

  return M2CryptoSSLTransport(ctx)





class SeattleClearinghouseError(Exception):
  """Base class for exceptions raised by the SeattleClearinghouseClient."""
  

class CommunicationError(SeattleClearinghouseError):
  """
  Indicates that XMLRPC communication failed.
  """
  
  
class InternalError(SeattleClearinghouseError):
  """
  Indicates an unexpected error occurred, probably either a bug in this
  client or a bug in SeattleClearinghouse.
  """


class AuthenticationError(SeattleClearinghouseError):
  """Indicates an authentication failure (invalid username and/or API key)."""
  def __init__(self, msg=None):
    if msg is None:
      msg = "Authentication failed. Invalid username and/or API key."
    SeattleClearinghouseError.__init__(self, msg)


class InvalidRequestError(SeattleClearinghouseError):
  """Indicates that the request is invalid."""


class NotEnoughCreditsError(SeattleClearinghouseError):
  """
  Indicates that the requested operation requires more vessel credits to
  be available then the account currently has.
  """
  
  
class UnableToAcquireResourcesError(SeattleClearinghouseError):
  """
  Indicates that the requested operation failed because SeattleClearinghouse was unable
  to acquire the requested resources.
  """



"""
<Program Name>
  admin.py

<Started>
  July 31, 2008

<Author>
  Justin Samuel

<Purpose>
  This module provides classes that tell django how to represent the models
  in the admin area of the website.

  See http://docs.djangoproject.com/en/dev/ref/contrib/admin/
"""

from seattlegeni.common.api import maindb

from seattlegeni.website.control.models import Donation
from seattlegeni.website.control.models import GeniUser
from seattlegeni.website.control.models import Node
from seattlegeni.website.control.models import Vessel
from seattlegeni.website.control.models import VesselPort
from seattlegeni.website.control.models import VesselUserAccessMap
from seattlegeni.website.control.models import ActionLogEvent
from seattlegeni.website.control.models import ActionLogVesselDetails

from django.contrib import admin
from django.contrib.auth.forms import AdminPasswordChangeForm
from django.contrib import messages
from django.core.exceptions import PermissionDenied
from django.http import HttpResponseRedirect, Http404
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.html import escape
from django.utils.translation import ugettext, ugettext_lazy as _




class GeniUserAdmin(admin.ModelAdmin):
  """Customized admin view of the GeniUser model."""

  # Use django's admin change password form
  change_password_form = AdminPasswordChangeForm

  list_display = ["username", "affiliation", "email", "free_vessel_credits", 
                  "usable_vessel_port", "date_created", "is_staff",
                  "is_superuser"]
  list_filter = ["free_vessel_credits", "date_created", "is_staff",
                 "is_superuser", "usable_vessel_port"]
  # Don't display the private key in the details form.
  exclude = ['user_privkey']
  search_fields = ["username", "user_pubkey", "donor_pubkey", "email",
                   "affiliation"]
  ordering = ["-date_created"]
  


  def get_urls(self):
    from django.conf.urls.defaults import patterns

    # Assign handler for the password change url
    return patterns('',
      (r'^(\d+)/password/$',
        self.admin_site.admin_view(self.user_change_password))
    ) + super(GeniUserAdmin, self).get_urls()



  def user_change_password(self, request, id):
    if not self.has_change_permission(request):
      raise PermissionDenied
    user = get_object_or_404(self.model, pk=id)

    if request.method == 'POST':
      # Form was submitted, if form data is valid, update user password
      form = self.change_password_form(user, request.POST)
      if form.is_valid():
        new_user = form.save()
        msg = ugettext('Password changed successfully.')
        messages.success(request, msg)
        return HttpResponseRedirect('..')
    else:
      form = self.change_password_form(user)

    # AdminPasswordChangeForm has 2 fields: Password and Password (again)
    fieldsets = [(None, {'fields': form.base_fields.keys()})]
    adminForm = admin.helpers.AdminForm(form, fieldsets, {})

    # Escape html and js special chars and set strings according to locale
    # See django/contrib/admin/templatetags/admin_modify.py for meanings of
    # following tags. They are mostly used for deciding what elements are to be
    # shown on the form.
    return render_to_response('admin/auth/user/change_password.html', {
        'title': _('Change password: %s') % escape(user.username),
        'adminForm': adminForm,
        'form': form,
        'is_popup': '_popup' in request.REQUEST,
        'add': True,
        'change': False,
        'has_delete_permission': False,
        'has_change_permission': True,
        'has_absolute_url': False,
        'opts': self.model._meta,
        'original': user,
        'save_as': False,
        'show_save': True,
        'root_path': self.admin_site.root_path,
    }, context_instance=RequestContext(request))





def partial_node_identifier(node):
  """Used by class NodeAdmin."""
  return node.node_identifier[:16]


def is_ok(node):
  """
  Used by class NodeAdmin. We use this because otherwise the green checkmark
  indicating broken is confusing. So, instead we have the field be the
  "not broken" field where a green checkmark means it's not broken.
  """
  return not node.is_broken
# Set a boolean attribute of the function itself which tells django to use the
# boolean icons to represent this field.
is_ok.boolean = True
# Set an admin_order_field of the function which tells django that what is
# represented by this column in the admin area is a field that can be ordered.
is_ok.admin_order_field = 'is_broken'


def donor(node):
  """
  Display a list of users who made donations from the node (which will always
  be one at the moment).
  """
  donation_list = maindb.get_donations_from_node(node)
  donor_names_list = []
  for donation in donation_list:
    donor_names_list.append(donation.donor.username)
  return ",".join(donor_names_list)
  

class NodeAdmin(admin.ModelAdmin):
  """Customized admin view of the Node model."""
  list_display = [partial_node_identifier, donor, "last_known_ip",
                  "last_known_port", "last_known_version", "is_active", is_ok,
                  "extra_vessel_name", "date_last_contacted", "date_created"]
  list_filter = ["last_known_port", "last_known_version", "is_active",
                 "is_broken", "extra_vessel_name", "date_last_contacted",
                 "date_created"]
  search_fields = ["node_identifier", "last_known_ip", "owner_pubkey"]
  ordering = ["-date_created"]





class DonationAdmin(admin.ModelAdmin):
  """Customized admin view of the Donation model."""
  list_display = ["node", "donor", "date_created"]
  list_filter = ["date_created"]
  search_fields = ["node__node_identifier", "donor__username"]
  ordering = ["-date_created"]





class VesselAdmin(admin.ModelAdmin):
  """Customized admin view of the Vessel model."""
  list_display = ["node", "name", "acquired_by_user", "date_acquired",
                  "date_expires", "is_dirty", "date_created"]
  list_filter = ["date_acquired", "date_expires", "is_dirty", "date_created"]
  search_fields = ["node__node_identifier", "node__last_known_ip", "name",
                   "acquired_by_user__username"]
  ordering = ["-date_acquired"]





class VesselPortAdmin(admin.ModelAdmin):
  """Customized admin view of the VesselPort model."""
  list_display = ["vessel", "port"]
  list_filter = []
  search_fields = ["vessel__node__node_identifier",
                   "vessel__node__last_known_ip", "port"]





class VesselUserAccessMapAdmin(admin.ModelAdmin):
  """Customized admin view of the VesselUserAccessMap model."""
  list_display = ["vessel", "user", "date_created"]
  list_filter = ["date_created"]
  search_fields = ["vessel__node__node_identifier",
                   "vessel__node__last_known_ip", "user__username"]
  ordering = ["-date_created"]





class ActionLogEventAdmin(admin.ModelAdmin):
  """Customized admin view of the ActionLogEvent model."""
  
  list_display = ["function_name", "user", "second_arg", "third_arg",
                  "was_successful", "message", "vessel_count", "date_started",
                  "completion_time"]
  list_filter = ["function_name", "user", "third_arg", "was_successful",
                 "vessel_count", "date_started"]
  search_fields = ["function_name", "user"]
  ordering = ["-date_started"]





class ActionLogVesselDetailsAdmin(admin.ModelAdmin):
  """Customized admin view of the ActionLogVesselDetails model."""
  
  list_display = ["event", "node", "node_address", "node_port", "vessel_name"]
  search_fields = ["node_address"]
  ordering = ["-event"]





# Register/associate each custom admin view defined above with the
# corresponding model defined in seattlegeni.website.control.models
admin.site.register(GeniUser, GeniUserAdmin)
admin.site.register(Node, NodeAdmin)
admin.site.register(Donation, DonationAdmin)
admin.site.register(Vessel, VesselAdmin)
admin.site.register(VesselPort, VesselPortAdmin)
admin.site.register(VesselUserAccessMap, VesselUserAccessMapAdmin)
admin.site.register(ActionLogEvent, ActionLogEventAdmin)
admin.site.register(ActionLogVesselDetails, ActionLogVesselDetailsAdmin)



"""
<Program>
  views.py

<Started>
  6 July 2009

<Author>
  Jason Chen
  Justin Samuel

<Purpose>
  This file defines all of the public SeattleGeni XMLRPC functions. When the
  SeattleGENI XMLRPC API changes, this file will generally always need to be
  modified.  

  To create a new xmlrpc function, just create a new method in the
  PublicXMLRPCFunctions class below. All public methods in that class
  are automatically registered with the xmlrpc dispatcher.

  The functions defined here should generally be making calls to the
  controller interface to perform any work or retrieve data.

  Be sure to use the following decorators with each function, in this order:

  @staticmethod -- this is a python decorator that prevents 'self'
                   from being passed as the first argument.
  @log_function_call -- this is our own decorator for logging purposes.
"""

# To send the admins emails when there's an unhandled exception.
import django.core.mail 

import random
import string
import traceback

# Used for raising xmlrpc faults
import xmlrpclib

from seattlegeni.website import settings

# Make available all of our own standard exceptions.
from seattlegeni.common.exceptions import *

from seattlegeni.common.util import assertions
from seattlegeni.common.util import log

# This is the logging decorator we use.
from seattlegeni.common.util.decorators import log_function_call
from seattlegeni.common.util.decorators import log_function_call_without_first_argument

# All of the work that needs to be done is passed through the controller interface.
from seattlegeni.website.control import interface

from seattle.repyportability import *
add_dy_support(locals())

dy_import_module_symbols("rsa.r2py")





# The number of bytes of random padding data, not including the "!" separator,
# which is included in the encrypted API key.
# Don't change this without changing the API spec and the xmlrpc client.
ENCRYPTED_API_KEY_PADDING_BYTES = 20



# XMLRPC Fault Code Constants
FAULTCODE_INTERNALERROR = 100
FAULTCODE_AUTHERROR = 101
FAULTCODE_INVALIDREQUEST = 102
FAULTCODE_NOTENOUGHCREDITS = 103
# 104 used to be used for "private key doesn't exist".
FAULTCODE_UNABLETOACQUIRE = 105




class PublicXMLRPCFunctions(object):
  """
  All public functions of this class are automatically exposed as part of the
  xmlrpc interface.
  
  Each method should be sure to check the user input and return useful errors
  to the client if the input is invalid. Note that raising an AssertionError
  (e.g. through a call to an assert_* method) won't be sufficient, as those
  should only indicate something going wrong in our code. 
  """

  def _dispatch(self, method, args):
    """
    We provide a _dispatch function (which SimpleXMLRPCServer looks for and
    uses) so that we can log exceptions due to our programming errors within
    seattlegeni as well to detect incorrect usage by clients.
    """
      
    try:
      # Get the requested function (making sure it exists).
      try:
        func = getattr(self, method)
      except AttributeError:
        raise InvalidRequestError("The requested method '" + method + "' doesn't exist.")
      
      # Call the requested function.
      return func(*args)
    
    except InvalidRequestError:
      log.error("The xmlrpc server was used incorrectly: " + traceback.format_exc())
      raise
    
    except xmlrpclib.Fault:
      # A xmlrpc Fault was intentionally raised by the code in this module.
      raise
    
    except Exception, e:
      # We assume all other exceptions are bugs in our code.

      # We use the log message as the basis for the email message, as well.
      logmessage = "Internal error while handling an xmlrpc request: " + traceback.format_exc()
      log.critical(logmessage)

      # Normally django will send an email to the ADMINS defined in settings.py
      # when an exception occurs. However, our xmlrpc dispatcher will turn this
      # into a Fault that is returned to the client. So, django won't see it as
      # an uncaught exception. Therefore, we have to send it ourselves.
      if not settings.DEBUG:
        subject = "Error handling xmlrpc request '" + method + "': " + str(type(e)) + " " + str(e)
        
        emailmessage = logmessage + "\n\n"
        emailmessage += "XMLRPC method called: " + method + "\n"
        
        # If the first argument looks like auth info, don't include the
        # api_key in the email we send. Otherwise, include all the args.
        # We wrap this in a try block just in case we screw this up we want to
        # be sure we get an email still.
        try:
          if len(args) > 0 and isinstance(args[0], dict) and "username" in args[0]:
            emailmessage += "Username: " + str(args[0]["username"]) + "\n"
            if len(args) > 1:
              emailmessage += "Non-auth arguments: " + str(args[1:]) + "\n"
            else:
              emailmessage += "There were no non-auth arguments." + "\n"
          else:
            emailmessage += "Arguments: " + str(args) + "\n"
        except:
          pass
          
        # Send an email to the addresses listed in settings.ADMINS
        django.core.mail.mail_admins(subject, emailmessage)
      
      # It's not unlikely that the user ends up seeing this message, so we
      # are careful about what the content of the message is. We don't
      # include the exception trace.
      raise xmlrpclib.Fault(FAULTCODE_INTERNALERROR, "Internal error while handling the xmlrpc request.")
      


  @staticmethod
  @log_function_call
  def acquire_resources(auth, rspec):
    """
    <Purpose>
      Acquires resources for users over XMLRPC. 
    <Arguments>
      auth
        An authorization dict.
      rspec
        A resource specification dict of the form {'rspec_type':type, 'number_of_nodes':num}
    <Exceptions>
      Raises xmlrpclib Fault objects:
        FAULTCODE_INTERNALERROR for internal errors.
        FAULTCODE_INVALIDREQUEST for bad user input.
        FAULTCODE_NOTENOUGHCREDITS if user has insufficient vessel credits to complete request.
    <Returns>
      A list of 'info' dictionaries, each 'infodict' contains acquired vessel info.
    """
    geni_user = _auth(auth)
    
    if not isinstance(rspec, dict):
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "rspec is an invalid data type.")
    
    try:
      resource_type = rspec['rspec_type']
    except KeyError:
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "rspec is missing rspec_type")
    
    try:
      num_vessels = rspec['number_of_nodes']
    except KeyError:
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "rspec is missing number_of_nodes")
    
    acquired_vessels = []
    
    # validate rspec data
    if not isinstance(resource_type, str) or not isinstance(num_vessels, int):
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "rspec has invalid data types.")
    
    if resource_type not in ['wan', 'lan', 'nat', 'random'] or num_vessels < 1:
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "rspec has invalid values.")
      
    # The interface.acquire_vessels() call expects 'rand' instead of 'random'.
    if resource_type == 'random':
      resource_type = 'rand'
    
    try:
      acquired_vessels = interface.acquire_vessels(geni_user, num_vessels, resource_type)
    except UnableToAcquireResourcesError, err:
      raise xmlrpclib.Fault(FAULTCODE_UNABLETOACQUIRE, "Unable to fulfill vessel acquire request at this given time. Details: " + str(err))
    except InsufficientUserResourcesError, err:
      raise xmlrpclib.Fault(FAULTCODE_NOTENOUGHCREDITS, "You do not have enough vessel credits to acquire the number of vessels requested.")
    
    # since acquire_vessels returns a list of Vessel objects, we
    # need to convert them into a list of 'info' dictionaries.
    return interface.get_vessel_infodict_list(acquired_vessels)



  @staticmethod
  @log_function_call
  def acquire_specific_vessels(auth, vesselhandle_list):
    """
    <Purpose>
      Acquires specific vessels for a user. This will be best effort. Zero or
      more of the vessels may be acquired. The list, however, cannot include
      more vessels than the maximum number the user is allowed to acquire.
    <Arguments>
      auth
        An authorization dict.
      vesselhandle_list
        A list of vessel handles.
    <Exceptions>
      Raises xmlrpclib Fault objects:
        FAULTCODE_INTERNALERROR for internal errors.
        FAULTCODE_INVALIDREQUEST for bad user input.
        FAULTCODE_NOTENOUGHCREDITS if user has insufficient vessel credits to complete request.
    <Returns>
      A list of 'info' dictionaries, each 'infodict' contains acquired vessel info.
    """
    geni_user = _auth(auth)
    
    if not isinstance(vesselhandle_list, list):
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "Invalid data type for handle list.")
    
    # Remove duplicates to avoid race condition where the database is
    # updated simultaneously for the same vessel, raising a duplicate
    # key error.
    vesselhandle_list = list(set(vesselhandle_list))

    # since we're given a list of vessel 'handles', we need to convert them to a 
    # list of actual Vessel objects; as release_vessels_of_user expects Vessel objs.
    try:
      list_of_vessel_objs = interface.get_vessel_list(vesselhandle_list)
    except DoesNotExistError, err:
      # given handle refers to a non-existant vessel
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    except InvalidRequestError, err:
      # A handle is of an invalid format or the list of vessels is empty.
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    
    try:
      acquired_vessels = interface.acquire_specific_vessels(geni_user, list_of_vessel_objs)
    except InvalidRequestError, err:
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    except InsufficientUserResourcesError, err:
      raise xmlrpclib.Fault(FAULTCODE_NOTENOUGHCREDITS, "You do not have enough " + 
                            "vessel credits to acquire the number of vessels requested.")
    
    # Convert the list of vessel objects into a list of 'info' dictionaries.
    return interface.get_vessel_infodict_list(acquired_vessels)



  @staticmethod
  @log_function_call
  def release_resources(auth, vesselhandle_list):
    """
    <Purpose>
      Release resources for a user over XMLRPC.
    <Arguments>
      auth
        An authorization dict.
      vesselhandle_list
        A list of vessel handles
    <Exceptions>
      Raises xmlrpclib Fault objects:
        FAULTCODE_INVALIDREQUEST if a user provides invalid vessel handles.
    <Returns>
      0 on success. Raises a fault otherwise.
    """
    geni_user = _auth(auth)
  
    if not isinstance(vesselhandle_list, list):
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "Invalid data type for handle list.")
    
    # since we're given a list of vessel 'handles', we need to convert them to a 
    # list of actual Vessel objects; as release_vessels_of_user expects Vessel objs.
    try:
      list_of_vessel_objs = interface.get_vessel_list(vesselhandle_list)
    except DoesNotExistError, err:
      # given handle refers to a non-existant vessel
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    except InvalidRequestError, err:
      # A handle is of an invalid format or the list of vessels is empty.
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    
    try:
      interface.release_vessels(geni_user, list_of_vessel_objs)
    except InvalidRequestError, err:
      # vessel exists but isn't valid for you to use.
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    
    return 0



  @staticmethod
  @log_function_call
  def renew_resources(auth, vesselhandle_list):
    """
    <Purpose>
      Renew resources for a user over XMLRPC. Renewal changes the expiration
      time to the maximum allowed.
    <Arguments>
      auth
        An authorization dict.
      vesselhandle_list
        A list of vessel handles
    <Exceptions>
      Raises xmlrpclib Fault objects with fault codes:
        FAULTCODE_INVALIDREQUEST if a user provides invalid vessel handles.
        FAULTCODE_NOTENOUGHCREDITS if user has insufficient vessel credits to complete request.
    <Returns>
      0 on success. Raises a fault otherwise.
    """
    geni_user = _auth(auth)
  
    if not isinstance(vesselhandle_list, list):
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "Invalid data type for handle list.")
    
    for handle in vesselhandle_list:
      if not isinstance(handle, str):
        raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, 
                              "Invalid data type for handle. Expected str, received " + str(type(handle)))

    # since we're given a list of vessel 'handles', we need to convert them to a 
    # list of actual Vessel objects.
    try:
      list_of_vessel_objs = interface.get_vessel_list(vesselhandle_list)
    except DoesNotExistError, err:
      # The handle refers to a non-existent vessel.
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    except InvalidRequestError, err:
      # A handle is of an invalid format or the list of vessels is empty.
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    
    try:
      interface.renew_vessels(geni_user, list_of_vessel_objs)
    except InvalidRequestError, err:
      # The vessel exists but isn't valid for this user to use.
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, str(err))
    except InsufficientUserResourcesError, err:
      message = "Vessels cannot be renewed because you are currently"
      message += " over your vessel credit limit: "
      message += str(err)
      raise xmlrpclib.Fault(FAULTCODE_NOTENOUGHCREDITS, message)
    
    return 0

  
  
  @staticmethod
  @log_function_call
  def get_resource_info(auth):
    """
    <Purpose>
      Gets a user's acquired vessels over XMLRPC.
    <Arguments>
      auth
        An authorization dict.
    <Exceptions>
      None.
    <Returns>
      A list of 'info' dictionaries, each 'infodict' contains vessel info.
    """
    geni_user = _auth(auth)
    user_vessels = interface.get_acquired_vessels(geni_user)
    return interface.get_vessel_infodict_list(user_vessels)
  
  
  
  @staticmethod
  @log_function_call
  def get_account_info(auth):
    """
    <Purpose>
      Gets a user's account info for a client over XMLRPC.
    <Arguments>
      auth
        An authorization dict.
    <Exceptions>
      None.
    <Returns>
      A dictionary containing account info.
    """
    geni_user = _auth(auth)
    user_port = geni_user.usable_vessel_port
    user_name = geni_user.username
    max_vessels = interface.get_total_vessel_credits(geni_user)
    user_affiliation = geni_user.affiliation
    infodict = {'user_port':user_port, 'user_name':user_name,
                'max_vessels':max_vessels, 'user_affiliation':user_affiliation}
    return infodict
  
  
  
  @staticmethod
  @log_function_call
  def get_public_key(auth):
    # Gets a user's public key.
    geni_user = _auth(auth)
    return geni_user.user_pubkey



  @staticmethod
  @log_function_call
  def get_encrypted_api_key(username):
    """
    <Purpose>
      Retrieve the account's API key encrypted with the account's public key.
      This provides the holder of an account's private key a means of obtaining
      the SeattleGENI API key for the account. A side effect is that this also
      provide a means of determining whether an account exists on seattlegeni.
      However, we do not protect against such querying here because the same
      can be determined in a handful of other ways through the html interface.
      
      This approach of obtaining the encrypted api key can be removed later if
      we implement a way to sign requests.
    <Arguments>
      username
        The username of the account whose encrypted API key we want.
    <Exceptions>
      Raises xmlrpclib Fault objects with fault codes:
        FAULTCODE_INVALIDREQUEST if a the provided username is not a string
          specifying a valid (existing) account username.
    <Returns>
      An string that represents DATA encrypted by (seattlelib) rsa.r2py's
      rsa_encrypt() function. DATA is a string that is the concatenation of
      a random string of 0-9a-fA-F of fixed length, an exclamation mark, and
      the API key.
      
      For example, DATA may be the following string:
        l28yDKLQGqhqdfDquDq0433!AD98OF2308Q9RYFHDHKJAC
      where
        AD98OF2308Q9RYFHDHKJAC
      is the API key.
      
      Note that this random data being prepended is just a dirty workaround for
      the lack of random padding being used by the encryption format offered by
      rsa.r2py or other repy libraries. This should be changed after a repy
      implementation of PKCS#1 is available.
    """
    try:
      assertions.assert_str(username)
    except AssertionError:
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "Username must be a string.")
    
    try:
      geniuser = interface.get_user_without_password(username)
    except DoesNotExistError:
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "Username does not exist.")
      
    # Don't change the number of bytes of random data without changing the API
    # spec and the xmlrpc client.
    randstring = ''.join(random.sample(string.letters + string.digits,
                                       ENCRYPTED_API_KEY_PADDING_BYTES)) 
    data = randstring + "!" + geniuser.api_key
    data = str(data) # make sure it's type str, not unicode, as required by rsa_encrypt
    user_pubkey_dict = rsa_string_to_publickey(geniuser.user_pubkey)
    encrypted_data = rsa_encrypt(data, user_pubkey_dict)
    # rsa_encrypt returns returns a string with an extra space in the front.
    encrypted_data = encrypted_data.strip() 
    # If the encrypted_data is more than one "block" long, something odd is
    # going on and there may be no padding applied to one or more encrypted
    # blocks.
    if ' ' in encrypted_data:
      raise InternalError("The encrypted_data to be returned by get_encrypted_api_key() " +
                          "was more than one block long. user: " + username + 
                          " encrypted_data: "+ encrypted_data)
    return encrypted_data



  @staticmethod
  @log_function_call_without_first_argument
  def regenerate_api_key(pwauth):
    """
    <Purpose>
      Regenerate a user's API key.This requires authenticating with the account
      password rather than the current API key.
    <Arguments>
      pwauth
        An authorization dict that includes a password instead of an apikey.
    <Exceptions>
      None.
    <Returns>
      The new API key.
    """
    geni_user = _pwauth(pwauth)
    return interface.regenerate_api_key(geni_user)



  @staticmethod
  @log_function_call_without_first_argument
  def set_public_key(pwauth, pubkeystring):
    """
    <Purpose>
      Sets the user account's public key. This requires authenticating with the
      account password rather than the current API key.
    <Arguments>
      pwauth
        An authorization dict that includes a password instead of an apikey.
      pubkeystring
        The account's new public key.
    <Exceptions>
      Raises xmlrpclib Fault Objects:
        FAULTCODE_INVALIDREQUEST if pubkey is invalid.
    <Returns>
      None.
    """
    geni_user = _pwauth(pwauth)
    try:
      interface.change_user_keys(geni_user, pubkeystring)
    except ValidationError, e:
      raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST, "Invalid public key: %s" % e)

    return 0



@log_function_call
def _auth(auth):
  """
  <Purpose>
    Internally used function that performs actual authorization.
  <Arguments>
    auth
      An authorization dict of the form {'username':username, 'api_key':api_key}
  <Exceptions>
    Raises xmlrpclib Fault Objects:
      FAULTCODE_INVALIDREQUEST if the auth dict is invalid.
      FAULTCODE_AUTHERROR if user auth fails.
  <Returns>
    On successful authentication, returns a geniuser object. Raises a fault otherwise.
  """
  if not isinstance(auth, dict):
    raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST,
                          "Auth dict must be a dictionary, not a " + str(type(auth)))
  
  try:
    username = auth['username']
    api_key = auth['api_key']
  except KeyError:
    raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST,
                          "Auth dict must contain both a 'username' and an 'api_key'.")
    
  try:
    geni_user = interface.get_user_with_api_key(username, api_key)
  except DoesNotExistError:
    raise xmlrpclib.Fault(FAULTCODE_AUTHERROR, "User auth failed.")
  
  return geni_user




def _pwauth(auth):
  """
  <Purpose>
    Internally used function that performs authorization based on the account
    password rather than the account api key.
  <Arguments>
    auth
      An authorization dict of the form {'username':username, 'password':password}
  <Exceptions>
    Raises xmlrpclib Fault Objects:
      FAULTCODE_INVALIDREQUEST if the auth dict is invalid.
      FAULTCODE_AUTHERROR if user auth fails.
  <Returns>
    On successful authentication, returns a geniuser object. Raises a fault otherwise.
  """
  if not isinstance(auth, dict):
    raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST,
                          "Auth dict must be a dictionary, not a " + str(type(auth)))
  
  try:
    username = auth['username']
    password = auth['password']
  except KeyError:
    raise xmlrpclib.Fault(FAULTCODE_INVALIDREQUEST,
                          "PasswordAuth dict must contain both a 'username' and an 'password'.")
    
  try:
    geni_user = interface.get_user_with_password(username, password)
  except DoesNotExistError:
    raise xmlrpclib.Fault(FAULTCODE_AUTHERROR, "User auth failed.")
  
  return geni_user



"""
<Program>
  test_donation_to_canonical.py

<Purpose>
  Test out the donation_to_canonical transition state. Basically
  tests out the node_transition_lib.py

<Authour>
  Monzur Muhammad
  monzum@cs.washington.edu

<Started>
  Aug 21, 2009
"""

# The seattlegeni testlib must be imported first.
from seattlegeni.tests import testlib

from seattlegeni.node_state_transitions import node_transition_lib

from seattlegeni.common.api import maindb

from seattlegeni.node_state_transitions.tests import mockutil



#vessel dictionary for this test
vessels_dict = {}
vessels_dict[mockutil.extra_vessel_name] = {"userkeys" : [node_transition_lib.acceptdonationpublickey],
                                   "ownerkey" : mockutil.donor_key,
                                   "ownerinfo" : "",
                                   "status" : "",
                                   "advertise" : True}





def setup_test():
  """
  <Purpose>
    Prepare everything in order to run the tests.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Efects>
    None

  <Return>
    None
  """

  testlib.setup_test_db()

  # Create a user who has the donation key.
  maindb.create_user(mockutil.testusername, "password", "example@example.com", "affiliation", 
                    "10 11", "2 2 2", mockutil.donor_key_str)

  # Setup all the mock functions
  mockutil.mock_nodemanager_get_node_info(mockutil.nodeid_key, "10.0test", vessels_dict)
  mockutil.mock_lockserver_calls()
  mockutil.mock_backend_generate_key([mockutil.per_node_key_str])
  mockutil.mock_nodemanager_get_vesselresources()
  mockutil.mock_transitionlib_do_advertise_lookup([mockutil.node_address])
  mockutil.mock_backend_set_vessel_owner_key()
  mockutil.mock_backend_split_vessel()
  mockutil.mock_backend_set_vessel_user_keylist([mockutil._mock_pubkey_to_string(
                                                node_transition_lib.canonicalpublickey)])
 



def run_success_test():
  """
  <Purpose>
    Run the test and make sure that the database was modified
    properly, the right keys were set, and generally all information
    is what was expected.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Effects>
    None

  <Return>
    None
  """

  transitionlist = []

  transitionlist.append((("donation_state", node_transition_lib.acceptdonationpublickey),
                        ("canonical_state", node_transition_lib.canonicalpublickey),
                         node_transition_lib.noop,
                         node_transition_lib.noop))

  (success_count, failure_count) = node_transition_lib.do_one_processnode_run(transitionlist, "startstatename", 1)[0]

  assert(success_count == 1)
  assert(failure_count == 0)

  assert_database_info()

  assert(mockutil.set_vessel_owner_key_call_count == 1)
  assert(mockutil.set_vessel_user_keylist_call_count == 1)  




def run_database_stituation_two_test():
  """
  <Purpose>
    The purpose of this test is to test the case where a record
    for the node has been registered in the database, but the 
    owner key on the node has not been changed and no donation
    record has been created.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Effects>
    None

  <Return>
    None
  """

  # Create a database entry for the node
  node_object = maindb.create_node(mockutil.nodeid_key_str, mockutil.node_ip, mockutil.node_port, "10.0test",
                                  False, mockutil.per_node_key_str, mockutil.extra_vessel_name)  

  transitionlist = []

  transitionlist.append((("donation_state", node_transition_lib.acceptdonationpublickey),
                        ("canonical_state", node_transition_lib.canonicalpublickey),
                         node_transition_lib.noop,
                         node_transition_lib.noop))

  print "Running test where database has record of node, no record of donation and node has donor key"

  (success_count, failure_count) = node_transition_lib.do_one_processnode_run(transitionlist, "startstatename", 1)[0]

  assert(success_count == 1)
  assert(failure_count == 0)

  assert_database_info()

  assert(mockutil.set_vessel_owner_key_call_count == 1)
  assert(mockutil.set_vessel_user_keylist_call_count == 1)




def run_database_stituation_three_test():
  """
  <Purpose>
    The purpose of this test is to test the case where a record
    for the node has been registered in the database and a 
    record for the donation has been registered, but the
    owner key on the node has not been changed.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Effects>
    None

  <Return>
    None
  """

  # Create a database entry for the node
  node_object = maindb.create_node(mockutil.nodeid_key_str, mockutil.node_ip, mockutil.node_port, "10.0test",
                                  False, mockutil.per_node_key_str, mockutil.extra_vessel_name)

  user_object = maindb.get_user(mockutil.testusername)

  # Create a donation for user
  maindb.create_donation(node_object, user_object, "Making a donation")

  transitionlist = []

  transitionlist.append((("donation_state", node_transition_lib.acceptdonationpublickey),
                        ("canonical_state", node_transition_lib.canonicalpublickey),
                         node_transition_lib.noop,
                         node_transition_lib.noop))

  print "Running test where database has record of node, has record of donation and node has donor key"

  (success_count, failure_count) = node_transition_lib.do_one_processnode_run(transitionlist, "startstatename", 1)[0]

  assert(success_count == 1)
  assert(failure_count == 0)

  assert_database_info()

  assert(mockutil.set_vessel_owner_key_call_count == 1)
  assert(mockutil.set_vessel_user_keylist_call_count == 1)





def run_multiple_donation_test():
  """
  <Purpose>
    The purpose of this test is to see how the transtion script would
    react if we provide a dictionary where a vessel is in the acceptdonation
    state but the node has already in the canonical state. This is possible
    if the node had two vessel in the acceptdonation state to start with.
    This is essentially if the node has two donations. Currently we don't handle
    multiple donations from the second node, so when we run the test, the success
    count should be 0 and the failure count should be 1. Also the name of the extra
    vessel should not change.

  <Arguments>
    None

  <Exceptions>
    None

  <Side Effects>
    None

  <Return>
    None
  """
 
  # Have the extra vessel in canonical form
  vessels_dict[mockutil.extra_vessel_name]["userkeys"] = [node_transition_lib.canonicalpublickey]

  # Have a second vessel in acceptdonation state. Essentially a second donation.
  vessels_dict["second_donation"] = {"userkeys" : [node_transition_lib.acceptdonationpublickey],
                                   "ownerkey" : mockutil.donor_key,
                                   "ownerinfo" : "",
                                   "status" : "",
                                   "advertise" : True}

  # Create a database entry for the node
  node_object = maindb.create_node(mockutil.nodeid_key_str, mockutil.node_ip, mockutil.node_port, "10.0test",
                                  False, mockutil.per_node_key_str, mockutil.extra_vessel_name)
  user_object = maindb.get_user(mockutil.testusername)
  # Create a donation for user
  maindb.create_donation(node_object, user_object, "Making a donation")
  
  # Register the new vesseldict so get_info returns this new modified vesseldict
  mockutil.mock_nodemanager_get_node_info(mockutil.nodeid_key, "10.0test", vessels_dict)




  transitionlist = []

  transitionlist.append((("donation_state", node_transition_lib.acceptdonationpublickey),
                        ("canonical_state", node_transition_lib.canonicalpublickey),
                         node_transition_lib.noop,
                         node_transition_lib.noop))

  print "Running test where database has record of node, no record of donation and node has donor key"

  (success_count, failure_count) = node_transition_lib.do_one_processnode_run(transitionlist, "startstatename", 1)[0]

  assert(success_count == 0)
  assert(failure_count == 1)

  assert_database_info()

  assert(mockutil.set_vessel_owner_key_call_count == 0)
  assert(mockutil.set_vessel_user_keylist_call_count == 0)





def assert_database_info():

  active_nodes_list = maindb.get_active_nodes()
  assert(len(active_nodes_list) == 0)
  
  testuser = maindb.get_user(mockutil.testusername)
    
  active_donations = maindb.get_donations_by_user(testuser)
  assert(len(active_donations) == 0)
  
  all_donations = maindb.get_donations_by_user(testuser, include_inactive_and_broken=True)
  assert(len(all_donations) == 1)
  
  node = all_donations[0].node
  
  assert(node.node_identifier == mockutil.nodeid_key_str)
  assert(node.last_known_ip == mockutil.node_ip)
  assert(node.last_known_port == mockutil.node_port)
  assert(node.extra_vessel_name == mockutil.extra_vessel_name)
  assert(node.owner_pubkey == mockutil.per_node_key_str)





def teardown_test():

  # Cleanup the test database.
  testlib.teardown_test_db()




if __name__ == "__main__":

  setup_test()
  try:
    run_success_test()
  finally:
    teardown_test()

  setup_test()
  try:
    run_database_stituation_two_test()
  finally:
    teardown_test()

  setup_test()
  try:
    run_database_stituation_three_test()
  finally:
    teardown_test()

  setup_test()
  try:
    run_multiple_donation_test()
  finally:
    teardown_test()

  



"""
<Program>
  logexception.py

<Started>
  24 August 2009

<Author>
  Justin Samuel

<Purpose>
  We register "middleware" with django to log exceptions. By default, django
  will send an email to the admins if DEBUG is False, but it will not log
  anything.

  Documenation on django middleware: http://www.djangobook.com/en/1.0/chapter15/
"""

import traceback

from seattlegeni.common.util import log

class LogExceptionMiddleware(object):
  
  def process_exception(self, request, exception):
    log.critical("An unhandled exception resulted from a request: " + traceback.format_exc())
    
    # Returning None indicates that default exception handling should be done.
    return None



from django.conf.urls.defaults import *

from django.conf import settings
from django.views.generic.simple import redirect_to
from django.views.generic import RedirectView
from django.shortcuts import render_to_response, redirect
# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

# We override the default error handler because we want to pass a RequestContext
# to the template so that it can know the MEDIA_URL and so look nice.
handler500 = 'seattlegeni.website.html.errorviews.internal_error'
# the format for urlpatterns is
# (regular exp, function, optional dictionary, optional name)
urlpatterns = patterns('',
    
    (r'^html/', include('seattlegeni.website.html.urls')),
    (r'^download/', include('seattlegeni.website.html.downloadurls')),
    (r'^xmlrpc', include('seattlegeni.website.xmlrpc.urls')),
    (r'^reports/', include('seattlegeni.website.reports.urls')),
    # OPenID/OAuth pages, RedirectView.as_view cuts off the complete/backend/ in the url
    url(r'^complete/(?P<backend>[^/]+)/error', RedirectView.as_view(url='/html/error')),
    url(r'^complete/(?P<backend>[^/]+)/associate_error', RedirectView.as_view(url='/html/associate_error')),
    url(r'^complete/(?P<backend>[^/]+)/profile', RedirectView.as_view(url='/html/profile')),
    #url(r'^complete/(?P<backend>[^/]+)/new_auto_register_user', RedirectView.as_view(url='/html/new_auto_register_user')),
    #Currently unused, we've disabled disconnected social accounts
    #url(r'^disconnect/(?P<backend>[^/]+)/profile', RedirectView.as_view(url='/html/profile')),
    (r'', include('social_auth.urls')),
    # Uncomment the admin/doc line below and add 'django.contrib.admindocs' 
    # to INSTALLED_APPS to enable admin documentation:
    # (r'^admin/doc/', include('django.contrib.admindocs.urls')),
    # Uncomment the next line to enable the admin:
    (r'^admin/', include(admin.site.urls)),
)

# If DEBUG is True, then this is for development rather than production. So,
# have django serve static files so apache isn't needed for development.
if settings.DEBUG:
  urlpatterns += patterns('',
      (r'^' + settings.MEDIA_URL[1:] + '(?P<path>.*)$', 'django.views.static.serve',
        {'document_root': settings.MEDIA_ROOT}),
  )



"""
<Program>
  transition_canonical_to_twopercent.py

<Purpose>
  The purpose of this program is to transition nodes from the
  canonical state to the twopercent state by bypassing
  through the movingto_twopercent state.

<Started>
  November 10, 2010

<Author>
  Monzur Muhammad
  monzum@cs.washington.edu

<Usage>
  Ensure that seattlegeni and seattle are in the PYTHONPATH. 
  Ensure that the database is setup properly and django settings
    are set correctly.

  python transition_canonical_to_twopercent.py 
"""


import os

import node_transition_lib






# The full path to the onepercentmanyevents.resources file, including the filename.
RESOURCES_TEMPLATE_FILE_PATH = os.path.join(os.path.dirname(__file__), "resource_files", "twopercent.resources")





def main():
  """
  <Purpose>
    The main function that calls the process_nodes_and_change_state() function
    in the node_transition_lib passing in the process and error functions.

  <Arguments>
    None
 
  <Exceptions>
    None

  <Side Effects>
    None
  """

  # Open and read the resource file that is necessary for twopercent vessels.
  # This will determine how the vessels will be split and how much resource 
  # will be allocated to each vessel.
  twopercent_resource_fd = file(RESOURCES_TEMPLATE_FILE_PATH)
  twopercent_resourcetemplate = twopercent_resource_fd.read()
  twopercent_resource_fd.close()
  
  # We are going to transition all the nodes that are in the canonical state
  # to the twopercent state. We are going to do this in three different 
  # state. First we are going to transition all the canonical state nodes
  # to the movingto_twopercent state with a no-op function. The reason for
  # this is, so if anything goes wrong, we can revert back.
  # In the second step we are going to attempt to move all the nodes in the
  # movingto_twopercent state to the twopercent state. The way to do this, is
  # we are going to split the vessels by giving each vessel the resources 
  # that are described in the resource template.
  # Next we are going to try to transition all the nodes in the 
  # movingto_twopercent state to the canonical state. Any nodes that failed 
  # to go to the twopercent are still stuck in the movingto_twopercent state,
  # and we want to move them back to the canonical state.

  # Variables that determine weather to mark a node inactive or not.
  mark_node_inactive = False
  mark_node_active = True

  state_function_arg_tuplelist = [
    ("canonical", "movingto_twopercent", node_transition_lib.noop, 
     node_transition_lib.noop, mark_node_inactive),

    ("movingto_twopercent", "twopercent", node_transition_lib.split_vessels, 
     node_transition_lib.noop, mark_node_active, twopercent_resourcetemplate),

    ("movingto_twopercent", "canonical", node_transition_lib.combine_vessels, 
     node_transition_lib.noop, mark_node_inactive)]
 
  sleeptime = 10
  process_name = "canonical_to_twopercent"
  parallel_instances = 10

  #call process_nodes_and_change_state() to start the node state transition
  node_transition_lib.process_nodes_and_change_state(state_function_arg_tuplelist, process_name, sleeptime, parallel_instances) 





if __name__ == '__main__':
  main()



#!/usr/bin/env python
"""
   Author: Justin Samuel

   Start Date: 3 July 2009
   
   This will script will run the unit tests in each of the files in the unit/
   directory.

   To create a new test case (one or more tests in a single file), create a new
   module in the unit/ directory. The module can have any name, but should try
   to be descriptive of what is in there. Then make a single class in that
   module named TheTestCase (best to just copy an existing module).

   The tests themselves (the methods in TheTestCase that should be run) should
   start with 'test'.

   Note: currently this script needs to be called from the same directory
   as it resides in. Use the 'run_tests.sh' script to make sure that happens
   no matter what your current working directory is.
"""

import os
import sys
import unittest
from glob import glob

sys.path.insert(0, "unit")

# Add to the path the directory that the lockserver module is in ('../').
# This assumes that the script will be run from the tests/ directory which
# is one directory below where the lockserver_daemon.py file is.
sys.path.append('..')

# Names of modules in the unit/ directory (excluding the '.py') whose test case
# won't be run.
ignoretests = []

# Don't set this. It is populated by the loop below.
tests = []

for file in glob("unit/*.py"):
    modulename = os.path.basename(file).split(".")[0]
    if modulename in ignoretests:
      continue
    module = __import__(modulename)
    tests.append(unittest.makeSuite(module.TheTestCase))


allTests = unittest.TestSuite(tests)
runner = unittest.TextTestRunner(verbosity=2)

testresult = runner.run(allTests)

if not testresult.wasSuccessful():
  sys.exit(1)

